# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Slot/Slot.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Slot/Slot
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Slot.Slot.build_geometry_active import build_geometry_active
except ImportError as error:
    build_geometry_active = error

try:
    from ..Methods.Slot.Slot.build_geometry_half_tooth import build_geometry_half_tooth
except ImportError as error:
    build_geometry_half_tooth = error

try:
    from ..Methods.Slot.Slot.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Slot.Slot.comp_angle_active_eq import comp_angle_active_eq
except ImportError as error:
    comp_angle_active_eq = error

try:
    from ..Methods.Slot.Slot.comp_angle_opening import comp_angle_opening
except ImportError as error:
    comp_angle_opening = error

try:
    from ..Methods.Slot.Slot.comp_height import comp_height
except ImportError as error:
    comp_height = error

try:
    from ..Methods.Slot.Slot.comp_height_active import comp_height_active
except ImportError as error:
    comp_height_active = error

try:
    from ..Methods.Slot.Slot.comp_height_opening import comp_height_opening
except ImportError as error:
    comp_height_opening = error

try:
    from ..Methods.Slot.Slot.comp_radius import comp_radius
except ImportError as error:
    comp_radius = error

try:
    from ..Methods.Slot.Slot.comp_radius_mid_active import comp_radius_mid_active
except ImportError as error:
    comp_radius_mid_active = error

try:
    from ..Methods.Slot.Slot.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Slot.Slot.comp_surface_active import comp_surface_active
except ImportError as error:
    comp_surface_active = error

try:
    from ..Methods.Slot.Slot.comp_width_opening import comp_width_opening
except ImportError as error:
    comp_width_opening = error

try:
    from ..Methods.Slot.Slot.get_is_stator import get_is_stator
except ImportError as error:
    get_is_stator = error

try:
    from ..Methods.Slot.Slot.get_name_lam import get_name_lam
except ImportError as error:
    get_name_lam = error

try:
    from ..Methods.Slot.Slot.get_Rbo import get_Rbo
except ImportError as error:
    get_Rbo = error

try:
    from ..Methods.Slot.Slot.get_surface import get_surface
except ImportError as error:
    get_surface = error

try:
    from ..Methods.Slot.Slot.get_surface_tooth import get_surface_tooth
except ImportError as error:
    get_surface_tooth = error

try:
    from ..Methods.Slot.Slot.is_outwards import is_outwards
except ImportError as error:
    is_outwards = error

try:
    from ..Methods.Slot.Slot.plot import plot
except ImportError as error:
    plot = error

try:
    from ..Methods.Slot.Slot.plot_active import plot_active
except ImportError as error:
    plot_active = error


from ._check import InitUnKnowClassError


class Slot(FrozenClass):
    """Generic class for slot (abstract)"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Slot.Slot.build_geometry_active
    if isinstance(build_geometry_active, ImportError):
        build_geometry_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method build_geometry_active: "
                    + str(build_geometry_active)
                )
            )
        )
    else:
        build_geometry_active = build_geometry_active
    # cf Methods.Slot.Slot.build_geometry_half_tooth
    if isinstance(build_geometry_half_tooth, ImportError):
        build_geometry_half_tooth = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method build_geometry_half_tooth: "
                    + str(build_geometry_half_tooth)
                )
            )
        )
    else:
        build_geometry_half_tooth = build_geometry_half_tooth
    # cf Methods.Slot.Slot.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Slot.Slot.comp_angle_active_eq
    if isinstance(comp_angle_active_eq, ImportError):
        comp_angle_active_eq = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method comp_angle_active_eq: "
                    + str(comp_angle_active_eq)
                )
            )
        )
    else:
        comp_angle_active_eq = comp_angle_active_eq
    # cf Methods.Slot.Slot.comp_angle_opening
    if isinstance(comp_angle_opening, ImportError):
        comp_angle_opening = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method comp_angle_opening: "
                    + str(comp_angle_opening)
                )
            )
        )
    else:
        comp_angle_opening = comp_angle_opening
    # cf Methods.Slot.Slot.comp_height
    if isinstance(comp_height, ImportError):
        comp_height = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method comp_height: " + str(comp_height))
            )
        )
    else:
        comp_height = comp_height
    # cf Methods.Slot.Slot.comp_height_active
    if isinstance(comp_height_active, ImportError):
        comp_height_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method comp_height_active: "
                    + str(comp_height_active)
                )
            )
        )
    else:
        comp_height_active = comp_height_active
    # cf Methods.Slot.Slot.comp_height_opening
    if isinstance(comp_height_opening, ImportError):
        comp_height_opening = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method comp_height_opening: "
                    + str(comp_height_opening)
                )
            )
        )
    else:
        comp_height_opening = comp_height_opening
    # cf Methods.Slot.Slot.comp_radius
    if isinstance(comp_radius, ImportError):
        comp_radius = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method comp_radius: " + str(comp_radius))
            )
        )
    else:
        comp_radius = comp_radius
    # cf Methods.Slot.Slot.comp_radius_mid_active
    if isinstance(comp_radius_mid_active, ImportError):
        comp_radius_mid_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method comp_radius_mid_active: "
                    + str(comp_radius_mid_active)
                )
            )
        )
    else:
        comp_radius_mid_active = comp_radius_mid_active
    # cf Methods.Slot.Slot.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method comp_surface: " + str(comp_surface))
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Slot.Slot.comp_surface_active
    if isinstance(comp_surface_active, ImportError):
        comp_surface_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method comp_surface_active: "
                    + str(comp_surface_active)
                )
            )
        )
    else:
        comp_surface_active = comp_surface_active
    # cf Methods.Slot.Slot.comp_width_opening
    if isinstance(comp_width_opening, ImportError):
        comp_width_opening = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method comp_width_opening: "
                    + str(comp_width_opening)
                )
            )
        )
    else:
        comp_width_opening = comp_width_opening
    # cf Methods.Slot.Slot.get_is_stator
    if isinstance(get_is_stator, ImportError):
        get_is_stator = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method get_is_stator: " + str(get_is_stator)
                )
            )
        )
    else:
        get_is_stator = get_is_stator
    # cf Methods.Slot.Slot.get_name_lam
    if isinstance(get_name_lam, ImportError):
        get_name_lam = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method get_name_lam: " + str(get_name_lam))
            )
        )
    else:
        get_name_lam = get_name_lam
    # cf Methods.Slot.Slot.get_Rbo
    if isinstance(get_Rbo, ImportError):
        get_Rbo = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method get_Rbo: " + str(get_Rbo))
            )
        )
    else:
        get_Rbo = get_Rbo
    # cf Methods.Slot.Slot.get_surface
    if isinstance(get_surface, ImportError):
        get_surface = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method get_surface: " + str(get_surface))
            )
        )
    else:
        get_surface = get_surface
    # cf Methods.Slot.Slot.get_surface_tooth
    if isinstance(get_surface_tooth, ImportError):
        get_surface_tooth = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Slot method get_surface_tooth: " + str(get_surface_tooth)
                )
            )
        )
    else:
        get_surface_tooth = get_surface_tooth
    # cf Methods.Slot.Slot.is_outwards
    if isinstance(is_outwards, ImportError):
        is_outwards = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method is_outwards: " + str(is_outwards))
            )
        )
    else:
        is_outwards = is_outwards
    # cf Methods.Slot.Slot.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # cf Methods.Slot.Slot.plot_active
    if isinstance(plot_active, ImportError):
        plot_active = property(
            fget=lambda x: raise_(
                ImportError("Can't use Slot method plot_active: " + str(plot_active))
            )
        )
    else:
        plot_active = plot_active
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, Zs=36, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Zs" in list(init_dict.keys()):
                Zs = init_dict["Zs"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.Zs = Zs

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Slot_str = ""
        if self.parent is None:
            Slot_str += "parent = None " + linesep
        else:
            Slot_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Slot_str += "Zs = " + str(self.Zs) + linesep
        return Slot_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.Zs != self.Zs:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._Zs != self._Zs:
            diff_list.append(name + ".Zs")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.Zs)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Slot_dict = dict()
        Slot_dict["Zs"] = self.Zs
        # The class name is added to the dict for deserialisation purpose
        Slot_dict["__class__"] = "Slot"
        return Slot_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Zs = None

    def _get_Zs(self):
        """getter of Zs"""
        return self._Zs

    def _set_Zs(self, value):
        """setter of Zs"""
        check_var("Zs", value, "int", Vmin=0, Vmax=1000)
        self._Zs = value

    Zs = property(
        fget=_get_Zs,
        fset=_set_Zs,
        doc=u"""slot number

        :Type: int
        :min: 0
        :max: 1000
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/FluxLinkFEMM.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/FluxLinkFEMM
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .FluxLink import FluxLink

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.FluxLinkFEMM.comp_fluxlinkage import comp_fluxlinkage
except ImportError as error:
    comp_fluxlinkage = error

try:
    from ..Methods.Simulation.FluxLinkFEMM.solve_FEMM import solve_FEMM
except ImportError as error:
    solve_FEMM = error


from ._check import InitUnKnowClassError


class FluxLinkFEMM(FluxLink):
    """Electric module: Flux Linkage with FEMM"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.FluxLinkFEMM.comp_fluxlinkage
    if isinstance(comp_fluxlinkage, ImportError):
        comp_fluxlinkage = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use FluxLinkFEMM method comp_fluxlinkage: "
                    + str(comp_fluxlinkage)
                )
            )
        )
    else:
        comp_fluxlinkage = comp_fluxlinkage
    # cf Methods.Simulation.FluxLinkFEMM.solve_FEMM
    if isinstance(solve_FEMM, ImportError):
        solve_FEMM = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use FluxLinkFEMM method solve_FEMM: " + str(solve_FEMM)
                )
            )
        )
    else:
        solve_FEMM = solve_FEMM
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        FEMM_dict=-1,
        type_calc_leakage=0,
        is_sliding_band=True,
        is_periodicity_a=False,
        Nt_tot=5,
        Kgeo_fineness=0.5,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "FEMM_dict" in list(init_dict.keys()):
                FEMM_dict = init_dict["FEMM_dict"]
            if "type_calc_leakage" in list(init_dict.keys()):
                type_calc_leakage = init_dict["type_calc_leakage"]
            if "is_sliding_band" in list(init_dict.keys()):
                is_sliding_band = init_dict["is_sliding_band"]
            if "is_periodicity_a" in list(init_dict.keys()):
                is_periodicity_a = init_dict["is_periodicity_a"]
            if "Nt_tot" in list(init_dict.keys()):
                Nt_tot = init_dict["Nt_tot"]
            if "Kgeo_fineness" in list(init_dict.keys()):
                Kgeo_fineness = init_dict["Kgeo_fineness"]
        # Set the properties (value check and convertion are done in setter)
        self.FEMM_dict = FEMM_dict
        self.type_calc_leakage = type_calc_leakage
        self.is_sliding_band = is_sliding_band
        self.is_periodicity_a = is_periodicity_a
        self.Nt_tot = Nt_tot
        self.Kgeo_fineness = Kgeo_fineness
        # Call FluxLink init
        super(FluxLinkFEMM, self).__init__()
        # The class is frozen (in FluxLink init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        FluxLinkFEMM_str = ""
        # Get the properties inherited from FluxLink
        FluxLinkFEMM_str += super(FluxLinkFEMM, self).__str__()
        FluxLinkFEMM_str += "FEMM_dict = " + str(self.FEMM_dict) + linesep
        FluxLinkFEMM_str += (
            "type_calc_leakage = " + str(self.type_calc_leakage) + linesep
        )
        FluxLinkFEMM_str += "is_sliding_band = " + str(self.is_sliding_band) + linesep
        FluxLinkFEMM_str += "is_periodicity_a = " + str(self.is_periodicity_a) + linesep
        FluxLinkFEMM_str += "Nt_tot = " + str(self.Nt_tot) + linesep
        FluxLinkFEMM_str += "Kgeo_fineness = " + str(self.Kgeo_fineness) + linesep
        return FluxLinkFEMM_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from FluxLink
        if not super(FluxLinkFEMM, self).__eq__(other):
            return False
        if other.FEMM_dict != self.FEMM_dict:
            return False
        if other.type_calc_leakage != self.type_calc_leakage:
            return False
        if other.is_sliding_band != self.is_sliding_band:
            return False
        if other.is_periodicity_a != self.is_periodicity_a:
            return False
        if other.Nt_tot != self.Nt_tot:
            return False
        if other.Kgeo_fineness != self.Kgeo_fineness:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from FluxLink
        diff_list.extend(super(FluxLinkFEMM, self).compare(other, name=name))
        if other._FEMM_dict != self._FEMM_dict:
            diff_list.append(name + ".FEMM_dict")
        if other._type_calc_leakage != self._type_calc_leakage:
            diff_list.append(name + ".type_calc_leakage")
        if other._is_sliding_band != self._is_sliding_band:
            diff_list.append(name + ".is_sliding_band")
        if other._is_periodicity_a != self._is_periodicity_a:
            diff_list.append(name + ".is_periodicity_a")
        if other._Nt_tot != self._Nt_tot:
            diff_list.append(name + ".Nt_tot")
        if other._Kgeo_fineness != self._Kgeo_fineness:
            diff_list.append(name + ".Kgeo_fineness")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from FluxLink
        S += super(FluxLinkFEMM, self).__sizeof__()
        if self.FEMM_dict is not None:
            for key, value in self.FEMM_dict.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.type_calc_leakage)
        S += getsizeof(self.is_sliding_band)
        S += getsizeof(self.is_periodicity_a)
        S += getsizeof(self.Nt_tot)
        S += getsizeof(self.Kgeo_fineness)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from FluxLink
        FluxLinkFEMM_dict = super(FluxLinkFEMM, self).as_dict(**kwargs)
        FluxLinkFEMM_dict["FEMM_dict"] = (
            self.FEMM_dict.copy() if self.FEMM_dict is not None else None
        )
        FluxLinkFEMM_dict["type_calc_leakage"] = self.type_calc_leakage
        FluxLinkFEMM_dict["is_sliding_band"] = self.is_sliding_band
        FluxLinkFEMM_dict["is_periodicity_a"] = self.is_periodicity_a
        FluxLinkFEMM_dict["Nt_tot"] = self.Nt_tot
        FluxLinkFEMM_dict["Kgeo_fineness"] = self.Kgeo_fineness
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        FluxLinkFEMM_dict["__class__"] = "FluxLinkFEMM"
        return FluxLinkFEMM_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.FEMM_dict = None
        self.type_calc_leakage = None
        self.is_sliding_band = None
        self.is_periodicity_a = None
        self.Nt_tot = None
        self.Kgeo_fineness = None
        # Set to None the properties inherited from FluxLink
        super(FluxLinkFEMM, self)._set_None()

    def _get_FEMM_dict(self):
        """getter of FEMM_dict"""
        return self._FEMM_dict

    def _set_FEMM_dict(self, value):
        """setter of FEMM_dict"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("FEMM_dict", value, "dict")
        self._FEMM_dict = value

    FEMM_dict = property(
        fget=_get_FEMM_dict,
        fset=_set_FEMM_dict,
        doc=u"""To enforce user-defined values for FEMM main parameters 

        :Type: dict
        """,
    )

    def _get_type_calc_leakage(self):
        """getter of type_calc_leakage"""
        return self._type_calc_leakage

    def _set_type_calc_leakage(self, value):
        """setter of type_calc_leakage"""
        check_var("type_calc_leakage", value, "int", Vmin=0, Vmax=1)
        self._type_calc_leakage = value

    type_calc_leakage = property(
        fget=_get_type_calc_leakage,
        fset=_set_type_calc_leakage,
        doc=u"""0 no leakage calculation /  1 calculation using single slot 

        :Type: int
        :min: 0
        :max: 1
        """,
    )

    def _get_is_sliding_band(self):
        """getter of is_sliding_band"""
        return self._is_sliding_band

    def _set_is_sliding_band(self, value):
        """setter of is_sliding_band"""
        check_var("is_sliding_band", value, "bool")
        self._is_sliding_band = value

    is_sliding_band = property(
        fget=_get_is_sliding_band,
        fset=_set_is_sliding_band,
        doc=u"""0 to desactivate the sliding band

        :Type: bool
        """,
    )

    def _get_is_periodicity_a(self):
        """getter of is_periodicity_a"""
        return self._is_periodicity_a

    def _set_is_periodicity_a(self, value):
        """setter of is_periodicity_a"""
        check_var("is_periodicity_a", value, "bool")
        self._is_periodicity_a = value

    is_periodicity_a = property(
        fget=_get_is_periodicity_a,
        fset=_set_is_periodicity_a,
        doc=u"""True to take into account the spatial periodicity of the machine

        :Type: bool
        """,
    )

    def _get_Nt_tot(self):
        """getter of Nt_tot"""
        return self._Nt_tot

    def _set_Nt_tot(self, value):
        """setter of Nt_tot"""
        check_var("Nt_tot", value, "int")
        self._Nt_tot = value

    Nt_tot = property(
        fget=_get_Nt_tot,
        fset=_set_Nt_tot,
        doc=u"""Number of time steps for the FEMM simulation

        :Type: int
        """,
    )

    def _get_Kgeo_fineness(self):
        """getter of Kgeo_fineness"""
        return self._Kgeo_fineness

    def _set_Kgeo_fineness(self, value):
        """setter of Kgeo_fineness"""
        check_var("Kgeo_fineness", value, "float")
        self._Kgeo_fineness = value

    Kgeo_fineness = property(
        fget=_get_Kgeo_fineness,
        fset=_set_Kgeo_fineness,
        doc=u"""global coefficient to adjust geometry fineness in FEMM (0.5 : default , > 1 : finner , < 1 : less fine)

        :Type: float
        """,
    )



# -*- coding: utf-8 -*-
from numpy import array, nan, tile, newaxis, ones
from SciDataTool import DataTime, DataFreq, Data1D
from logging import getLogger

from ....Classes.SolutionData import SolutionData


def _comp_loss_sum(self, LossDensComps, area, k_freq):
    # get the axes
    axes_list = [axis.name for axis in LossDensComps.axes]
    freqs_idx = axes_list.index("freqs")
    comps_idx = axes_list.index("Components")

    # compute the sum over all frequencies and elements of the loss components
    comp_names = ["Hysteresis", "Eddy", "Excess"]
    alphas = [1, self.alpha_ed, self.alpha_ex]  # hyst. is propotional to freq.

    loss_dict = {}
    for comp_name in comp_names:
        axis_data = {"Components": comp_name}
        idx = LossDensComps.axes[comps_idx].values.tolist().index(comp_name)
        axes_list_ = [axis for axis in axes_list]
        axes_list_[comps_idx] += f"[{idx}]"

        data_dict = LossDensComps.get_along(*axes_list_, axis_data=axis_data)
        data_freq_sum = data_dict["LossDensComps"].sum(axis=freqs_idx)
        loss_dict[comp_name] = (area * data_freq_sum).sum()

    # compute the losses of the different speeds
    for comp_name, alpha in zip(comp_names, alphas):
        k = array(k_freq) ** alpha
        loss_dict[comp_name] = loss_dict[comp_name] * k

    # compute the sum of the components
    loss_sum = None
    for comp_name in comp_names:
        if loss_sum is None:
            loss_sum = loss_dict[comp_name]
        else:
            loss_sum = loss_dict[comp_name] + loss_sum

    return loss_sum


def comp_loss(self, output, part_label):
    """Compute the Losses"""
    # get logger
    logger = self.get_logger()

    # check inpurt
    if not "Stator" in part_label and not "Rotor" in part_label:
        logger.warning(
            f"LossModelBertotti.comp_loss(): 'part_label'"
            + f" {part_label} not implemented yet."
        )
        return None, None

    # get the simulation and the lamination
    simu = output.simu
    lam = simu.machine.get_lam_by_label(part_label)

    # get length, material and speed
    L1 = lam.L1
    mat_type = lam.mat_type
    rho = mat_type.struct.rho
    N0 = output.elec.N0
    group_name = part_label.lower() + " " + self.group  # TODO unifiy FEA names

    # setup meshsolution and solution list
    meshsolution = output.mag.meshsolution.get_group(group_name)

    # compute needed model parameter from material data
    success = self.comp_coeff_Bertotti(mat_type)
    if not success:
        logger.warning("LossModelBertotti: Unable to estimate model coefficents.")

    if success:
        # compute loss density
        LossDens, LossDensComps = self.comp_loss_density(meshsolution)

        # compute sum over frequencies
        axes_list = [axis.name for axis in LossDens.axes]
        freqs_idx = axes_list.index("freqs")

        loss_dens_freq_sum = LossDens.get_along(*axes_list)["LossDens"].sum(
            axis=freqs_idx
        )

        time = Data1D(name="time", unit="", values=array([0, 1]))
        # time = Data1D(name="time", unit="", values=array([0]), ) # TODO squeeze issue
        axes = [axis for axis in LossDens.axes if axis.name not in ["time", "freqs"]]

        # TODO utilize periodicity or use DataFreqs to reduce memory usage
        LossDensSum = DataTime(
            name="Losses sum",
            unit="W/kg",
            symbol="LossDensSum",
            axes=[time, *axes],
            values=tile(loss_dens_freq_sum, (2, 1)),
            # values=loss_freqs_sum[newaxis,:], # TODO squeeze issue
        )

        # Set the symmetry factor according to the machine
        if simu.mag.is_periodicity_a:
            sym, is_antiper_a, _, _ = output.get_machine_periodicity()
            sym *= is_antiper_a + 1
        else:
            sym = 1

        # compute the sum of the losses
        area = meshsolution.get_mesh().get_cell_area()
        N0_list = self.N0 if self.N0 else [N0]
        k_freq = [n / N0 for n in N0_list]

        Time = output.elec.Time
        Speed = Data1D(name="speed", unit="rpm", symbol="N0", values=N0_list)

        loss_sum = _comp_loss_sum(self, LossDensComps, area, k_freq)[newaxis, :]
        loss_sum = (
            loss_sum * ones((Time.get_length(), 1))[:, newaxis]
        )  # TODO use periodicity
        loss_sum *= L1 * rho * sym
        data = DataTime(
            name=self.name, unit="W", symbol="Loss", axes=[Time, Speed], values=loss_sum
        )
        if self.get_meshsolution:
            solution = []
            meshsolution.solution = solution
            solution.append(SolutionData(field=LossDens, label="LossDens"))
            solution.append(SolutionData(field=LossDensComps, label="LossDensComps"))
            solution.append(SolutionData(field=LossDensSum, label="LossDensSum"))
            return data, meshsolution
        else:
            return data, None
    else:
        return None, None



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/OutGeo.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/OutGeo
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError
from .OutGeoLam import OutGeoLam


class OutGeo(FrozenClass):
    """Gather the geometrical and the global outputs"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        stator=None,
        rotor=None,
        Wgap_mec=None,
        Wgap_mag=None,
        Rgap_mec=None,
        Lgap=None,
        logger_name="Pyleecan.OutGeo",
        angle_offset_initial=None,
        rot_dir=None,
        per_a=None,
        is_antiper_a=None,
        per_t=None,
        is_antiper_t=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "stator" in list(init_dict.keys()):
                stator = init_dict["stator"]
            if "rotor" in list(init_dict.keys()):
                rotor = init_dict["rotor"]
            if "Wgap_mec" in list(init_dict.keys()):
                Wgap_mec = init_dict["Wgap_mec"]
            if "Wgap_mag" in list(init_dict.keys()):
                Wgap_mag = init_dict["Wgap_mag"]
            if "Rgap_mec" in list(init_dict.keys()):
                Rgap_mec = init_dict["Rgap_mec"]
            if "Lgap" in list(init_dict.keys()):
                Lgap = init_dict["Lgap"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "angle_offset_initial" in list(init_dict.keys()):
                angle_offset_initial = init_dict["angle_offset_initial"]
            if "rot_dir" in list(init_dict.keys()):
                rot_dir = init_dict["rot_dir"]
            if "per_a" in list(init_dict.keys()):
                per_a = init_dict["per_a"]
            if "is_antiper_a" in list(init_dict.keys()):
                is_antiper_a = init_dict["is_antiper_a"]
            if "per_t" in list(init_dict.keys()):
                per_t = init_dict["per_t"]
            if "is_antiper_t" in list(init_dict.keys()):
                is_antiper_t = init_dict["is_antiper_t"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.stator = stator
        self.rotor = rotor
        self.Wgap_mec = Wgap_mec
        self.Wgap_mag = Wgap_mag
        self.Rgap_mec = Rgap_mec
        self.Lgap = Lgap
        self.logger_name = logger_name
        self.angle_offset_initial = angle_offset_initial
        self.rot_dir = rot_dir
        self.per_a = per_a
        self.is_antiper_a = is_antiper_a
        self.per_t = per_t
        self.is_antiper_t = is_antiper_t

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OutGeo_str = ""
        if self.parent is None:
            OutGeo_str += "parent = None " + linesep
        else:
            OutGeo_str += "parent = " + str(type(self.parent)) + " object" + linesep
        if self.stator is not None:
            tmp = self.stator.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            OutGeo_str += "stator = " + tmp
        else:
            OutGeo_str += "stator = None" + linesep + linesep
        if self.rotor is not None:
            tmp = self.rotor.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            OutGeo_str += "rotor = " + tmp
        else:
            OutGeo_str += "rotor = None" + linesep + linesep
        OutGeo_str += "Wgap_mec = " + str(self.Wgap_mec) + linesep
        OutGeo_str += "Wgap_mag = " + str(self.Wgap_mag) + linesep
        OutGeo_str += "Rgap_mec = " + str(self.Rgap_mec) + linesep
        OutGeo_str += "Lgap = " + str(self.Lgap) + linesep
        OutGeo_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        OutGeo_str += (
            "angle_offset_initial = " + str(self.angle_offset_initial) + linesep
        )
        OutGeo_str += "rot_dir = " + str(self.rot_dir) + linesep
        OutGeo_str += "per_a = " + str(self.per_a) + linesep
        OutGeo_str += "is_antiper_a = " + str(self.is_antiper_a) + linesep
        OutGeo_str += "per_t = " + str(self.per_t) + linesep
        OutGeo_str += "is_antiper_t = " + str(self.is_antiper_t) + linesep
        return OutGeo_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.stator != self.stator:
            return False
        if other.rotor != self.rotor:
            return False
        if other.Wgap_mec != self.Wgap_mec:
            return False
        if other.Wgap_mag != self.Wgap_mag:
            return False
        if other.Rgap_mec != self.Rgap_mec:
            return False
        if other.Lgap != self.Lgap:
            return False
        if other.logger_name != self.logger_name:
            return False
        if other.angle_offset_initial != self.angle_offset_initial:
            return False
        if other.rot_dir != self.rot_dir:
            return False
        if other.per_a != self.per_a:
            return False
        if other.is_antiper_a != self.is_antiper_a:
            return False
        if other.per_t != self.per_t:
            return False
        if other.is_antiper_t != self.is_antiper_t:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.stator is None and self.stator is not None) or (
            other.stator is not None and self.stator is None
        ):
            diff_list.append(name + ".stator None mismatch")
        elif self.stator is not None:
            diff_list.extend(self.stator.compare(other.stator, name=name + ".stator"))
        if (other.rotor is None and self.rotor is not None) or (
            other.rotor is not None and self.rotor is None
        ):
            diff_list.append(name + ".rotor None mismatch")
        elif self.rotor is not None:
            diff_list.extend(self.rotor.compare(other.rotor, name=name + ".rotor"))
        if other._Wgap_mec != self._Wgap_mec:
            diff_list.append(name + ".Wgap_mec")
        if other._Wgap_mag != self._Wgap_mag:
            diff_list.append(name + ".Wgap_mag")
        if other._Rgap_mec != self._Rgap_mec:
            diff_list.append(name + ".Rgap_mec")
        if other._Lgap != self._Lgap:
            diff_list.append(name + ".Lgap")
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        if other._angle_offset_initial != self._angle_offset_initial:
            diff_list.append(name + ".angle_offset_initial")
        if other._rot_dir != self._rot_dir:
            diff_list.append(name + ".rot_dir")
        if other._per_a != self._per_a:
            diff_list.append(name + ".per_a")
        if other._is_antiper_a != self._is_antiper_a:
            diff_list.append(name + ".is_antiper_a")
        if other._per_t != self._per_t:
            diff_list.append(name + ".per_t")
        if other._is_antiper_t != self._is_antiper_t:
            diff_list.append(name + ".is_antiper_t")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.stator)
        S += getsizeof(self.rotor)
        S += getsizeof(self.Wgap_mec)
        S += getsizeof(self.Wgap_mag)
        S += getsizeof(self.Rgap_mec)
        S += getsizeof(self.Lgap)
        S += getsizeof(self.logger_name)
        S += getsizeof(self.angle_offset_initial)
        S += getsizeof(self.rot_dir)
        S += getsizeof(self.per_a)
        S += getsizeof(self.is_antiper_a)
        S += getsizeof(self.per_t)
        S += getsizeof(self.is_antiper_t)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        OutGeo_dict = dict()
        if self.stator is None:
            OutGeo_dict["stator"] = None
        else:
            OutGeo_dict["stator"] = self.stator.as_dict(**kwargs)
        if self.rotor is None:
            OutGeo_dict["rotor"] = None
        else:
            OutGeo_dict["rotor"] = self.rotor.as_dict(**kwargs)
        OutGeo_dict["Wgap_mec"] = self.Wgap_mec
        OutGeo_dict["Wgap_mag"] = self.Wgap_mag
        OutGeo_dict["Rgap_mec"] = self.Rgap_mec
        OutGeo_dict["Lgap"] = self.Lgap
        OutGeo_dict["logger_name"] = self.logger_name
        OutGeo_dict["angle_offset_initial"] = self.angle_offset_initial
        OutGeo_dict["rot_dir"] = self.rot_dir
        OutGeo_dict["per_a"] = self.per_a
        OutGeo_dict["is_antiper_a"] = self.is_antiper_a
        OutGeo_dict["per_t"] = self.per_t
        OutGeo_dict["is_antiper_t"] = self.is_antiper_t
        # The class name is added to the dict for deserialisation purpose
        OutGeo_dict["__class__"] = "OutGeo"
        return OutGeo_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.stator is not None:
            self.stator._set_None()
        if self.rotor is not None:
            self.rotor._set_None()
        self.Wgap_mec = None
        self.Wgap_mag = None
        self.Rgap_mec = None
        self.Lgap = None
        self.logger_name = None
        self.angle_offset_initial = None
        self.rot_dir = None
        self.per_a = None
        self.is_antiper_a = None
        self.per_t = None
        self.is_antiper_t = None

    def _get_stator(self):
        """getter of stator"""
        return self._stator

    def _set_stator(self, value):
        """setter of stator"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "stator"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutGeoLam()
        check_var("stator", value, "OutGeoLam")
        self._stator = value

        if self._stator is not None:
            self._stator.parent = self

    stator = property(
        fget=_get_stator,
        fset=_set_stator,
        doc=u"""Geometry output of the stator

        :Type: OutGeoLam
        """,
    )

    def _get_rotor(self):
        """getter of rotor"""
        return self._rotor

    def _set_rotor(self, value):
        """setter of rotor"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "rotor"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutGeoLam()
        check_var("rotor", value, "OutGeoLam")
        self._rotor = value

        if self._rotor is not None:
            self._rotor.parent = self

    rotor = property(
        fget=_get_rotor,
        fset=_set_rotor,
        doc=u"""Geometry output of the rotor

        :Type: OutGeoLam
        """,
    )

    def _get_Wgap_mec(self):
        """getter of Wgap_mec"""
        return self._Wgap_mec

    def _set_Wgap_mec(self, value):
        """setter of Wgap_mec"""
        check_var("Wgap_mec", value, "float")
        self._Wgap_mec = value

    Wgap_mec = property(
        fget=_get_Wgap_mec,
        fset=_set_Wgap_mec,
        doc=u"""mechanical airgap width (minimal distance between the lamination including magnet)

        :Type: float
        """,
    )

    def _get_Wgap_mag(self):
        """getter of Wgap_mag"""
        return self._Wgap_mag

    def _set_Wgap_mag(self, value):
        """setter of Wgap_mag"""
        check_var("Wgap_mag", value, "float")
        self._Wgap_mag = value

    Wgap_mag = property(
        fget=_get_Wgap_mag,
        fset=_set_Wgap_mag,
        doc=u"""the magnetic airgap width (distance beetween the two Laminations bore radius)

        :Type: float
        """,
    )

    def _get_Rgap_mec(self):
        """getter of Rgap_mec"""
        return self._Rgap_mec

    def _set_Rgap_mec(self, value):
        """setter of Rgap_mec"""
        check_var("Rgap_mec", value, "float")
        self._Rgap_mec = value

    Rgap_mec = property(
        fget=_get_Rgap_mec,
        fset=_set_Rgap_mec,
        doc=u"""radius of the center of the mecanical airgap

        :Type: float
        """,
    )

    def _get_Lgap(self):
        """getter of Lgap"""
        return self._Lgap

    def _set_Lgap(self, value):
        """setter of Lgap"""
        check_var("Lgap", value, "float")
        self._Lgap = value

    Lgap = property(
        fget=_get_Lgap,
        fset=_set_Lgap,
        doc=u"""Airgap active length

        :Type: float
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )

    def _get_angle_offset_initial(self):
        """getter of angle_offset_initial"""
        return self._angle_offset_initial

    def _set_angle_offset_initial(self, value):
        """setter of angle_offset_initial"""
        check_var("angle_offset_initial", value, "float")
        self._angle_offset_initial = value

    angle_offset_initial = property(
        fget=_get_angle_offset_initial,
        fset=_set_angle_offset_initial,
        doc=u"""Difference between the d axis angle of the stator and the rotor

        :Type: float
        """,
    )

    def _get_rot_dir(self):
        """getter of rot_dir"""
        return self._rot_dir

    def _set_rot_dir(self, value):
        """setter of rot_dir"""
        check_var("rot_dir", value, "int", Vmin=-1, Vmax=1)
        self._rot_dir = value

    rot_dir = property(
        fget=_get_rot_dir,
        fset=_set_rot_dir,
        doc=u"""rotation direction of the magnetic field fundamental !! WARNING: rot_dir = -1 to have positive rotor rotating direction, i.e. rotor position moves towards positive angle

        :Type: int
        :min: -1
        :max: 1
        """,
    )

    def _get_per_a(self):
        """getter of per_a"""
        return self._per_a

    def _set_per_a(self, value):
        """setter of per_a"""
        check_var("per_a", value, "int")
        self._per_a = value

    per_a = property(
        fget=_get_per_a,
        fset=_set_per_a,
        doc=u"""Number of spatial periodicities of the machine

        :Type: int
        """,
    )

    def _get_is_antiper_a(self):
        """getter of is_antiper_a"""
        return self._is_antiper_a

    def _set_is_antiper_a(self, value):
        """setter of is_antiper_a"""
        check_var("is_antiper_a", value, "bool")
        self._is_antiper_a = value

    is_antiper_a = property(
        fget=_get_is_antiper_a,
        fset=_set_is_antiper_a,
        doc=u"""True if an spatial anti-periodicity is possible after the periodicities

        :Type: bool
        """,
    )

    def _get_per_t(self):
        """getter of per_t"""
        return self._per_t

    def _set_per_t(self, value):
        """setter of per_t"""
        check_var("per_t", value, "int")
        self._per_t = value

    per_t = property(
        fget=_get_per_t,
        fset=_set_per_t,
        doc=u"""Number of time periodicities of the machine

        :Type: int
        """,
    )

    def _get_is_antiper_t(self):
        """getter of is_antiper_t"""
        return self._is_antiper_t

    def _set_is_antiper_t(self, value):
        """setter of is_antiper_t"""
        check_var("is_antiper_t", value, "bool")
        self._is_antiper_t = value

    is_antiper_t = property(
        fget=_get_is_antiper_t,
        fset=_set_is_antiper_t,
        doc=u"""True if an time anti-periodicity is possible after the periodicities

        :Type: bool
        """,
    )



# -*- coding: utf-8 -*-
from pandas import read_excel, ExcelFile
from ....Methods.Import.ImportMatrixXls import XlsFileError
from os.path import isfile

from ....Functions.path_tools import abs_file_path


def get_data(self):
    """Return the object's matrix

    Parameters
    ----------
    self : ImportMatrixVal
        An ImportMatrixVal object

    Returns
    -------
    matrix: ndarray
        The object's matrix

    """
    file_path = abs_file_path(self.file_path, is_check=False)
    if not isfile(file_path):
        raise XlsFileError("ERROR: The xls file doesn't exist " + self.file_path)
    if self.usecols == "":
        self.usecols = None

    if self.is_allsheets:
        xl = ExcelFile("file.xlsx")
        Nsheets = len(xl.sheet_names)
        for i in range(Nsheets):
            df = read_excel(
                file_path, i, header=None, usecols=self.usecols, skiprows=self.skiprows
            )
            values = df.values
            # Extract axes
            if self.axes_colrows is not None:
                axes_dict = {}
                if len(axes_dict) == 1:
                    is_first = False
                else:
                    is_first = True
                for axis in self.axes_colrows:
                    if self.axes_colrows[axis] == "1":
                        if is_first:
                            axes_dict[axis] = values[0, 1:]
                            is_first = False
                        else:
                            axes_dict[axis] = values[0, :]
                        values = values[1:, :]
                    elif self.axes_colrows[axis] == "A":
                        if is_first:
                            axes_dict[axis] = values[1:, 0]
                            is_first = False
                        else:
                            axes_dict[axis] = values[:, 0]
                        values = values[:, 1:]
                    else:
                        raise Exception(
                            "ERROR: axes_colrows should contain either '1' or 'A'"
                        )

            # Transpose if necessary
            values = self.edit_matrix(values)

            # Store in 3D matrix
            if i == 0:
                values_3d = values[..., None]
            else:
                values_3d.append(values_3d, values[..., None], axis=-1)

        if self.axes_colrows is not None:
            return values_3d, axes_dict
        else:
            return values_3d

    else:

        df = read_excel(
            file_path,
            self.sheet,
            header=None,
            usecols=self.usecols,
            skiprows=self.skiprows,
        )
        values = df.values

        # Extract axes
        if self.axes_colrows is not None:
            axes_dict = {}
            if len(axes_dict) == 1:
                is_first = False
            else:
                is_first = True
            for axis in self.axes_colrows:
                if self.axes_colrows[axis] == "1":
                    if is_first:
                        axes_dict[axis] = values[0, 1:]
                        is_first = False
                    else:
                        axes_dict[axis] = values[0, :]
                    values = values[1:, :]
                elif self.axes_colrows[axis] == "A":
                    if is_first:
                        axes_dict[axis] = values[1:, 0]
                        is_first = False
                    else:
                        axes_dict[axis] = values[:, 0]
                    values = values[:, 1:]
                else:
                    raise Exception(
                        "ERROR: axes_colrows should contain either '1' or 'A'"
                    )
        else:
            axes_dict = None

        # Transpose if necessary
        values = self.edit_matrix(values)

        if self.axes_colrows is not None:
            return values, axes_dict
        else:
            return values



# -*- coding: utf-8 -*-

from ....Classes.LamSlot import LamSlot


def comp_masses(self):
    """Compute the Lamination masses

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object

    Returns
    -------
    M_dict: dict
        Lamination mass dictionnary (Mtot, Mlam, Mwind) [kg]
    """

    M_dict = LamSlot.comp_masses(self)

    if self.winding is not None:
        L_dict = self.comp_lengths_winding()
        Mwind = (
            self.winding.conductor.comp_surface_active()
            * self.winding.conductor.cond_mat.struct.rho
            * L_dict["Lwtot"]
        )
    else:
        Mwind = 0

    M_dict["Mtot"] += Mwind
    M_dict["Mwind"] = Mwind

    return M_dict



# -*- coding: utf-8 -*-
from ....Methods.Geometry.Arc3 import *


def check(self):
    """assert that the arc is correct (begin != end)

    Parameters
    ----------
    self : Arc3
        An Arc3 object

    Returns
    -------
    None

    Raises
    ------
    PointArc3Error
        The beginning point and the ending point of an Arc3
        can't be the same

    """

    if self.begin == self.end or (abs(self.begin) == 0 and abs(self.end) == 0):
        raise PointArc3Error(
            "The beginning point and the ending point of an " "Arc3 can't be the same"
        )



# -*- coding: utf-8 -*-
from numpy import exp, pi

from ...Classes.Arc1 import Arc1
from ...Classes.Circle import Circle
from ...Classes.Segment import Segment
from ...Classes.SurfLine import SurfLine


def get_sliding_band(sym, lam_int, lam_ext):
    """Returns  a list of surface in the airgap including the sliding band surface

    Parameters
    ----------
    sym: int
        Symmetry factor (1= full machine, 2= half of the machine...)
    Rgap_mec_int: float
        Internal lamination mechanic radius
    Rgap_mec_ext: float
        External lamination mechanic radius

    Returns
    -------
    surf_list: list
        List of surface in the airgap including the sliding band surface
    """

    Rgap_mec_int = lam_int.comp_radius_mec()
    Rgap_mec_ext = lam_ext.comp_radius_mec()
    Rgap_mag_int = lam_int.Rext
    Rgap_mag_ext = lam_ext.Rint
    Wgap_mec = Rgap_mec_ext - Rgap_mec_int
    W_sb = Wgap_mec / 3  # Width sliding band
    surf_list = list()
    if sym == 1:  # Complete machine
        # Bottom sliding band
        surf_list.append(
            Circle(
                center=0,
                radius=Rgap_mec_int + W_sb,
                label="Airgap",
                point_ref=(Rgap_mec_int + W_sb / 2) * exp(1j * pi / 2),
                line_label="sliding_line",
            )
        )
        # Top sliding band
        surf_list.append(
            Circle(
                center=0,
                radius=Rgap_mec_int + 2 * W_sb,
                label="Airgap",
                point_ref=(Rgap_mec_ext - W_sb / 2) * exp(1j * pi / 2),
                line_label="sliding_line",
            )
        )
        # Middle
        surf_list.append(
            SurfLine(
                line_list=[],
                point_ref=(Rgap_mec_int + W_sb * 3 / 2) * exp(1j * pi / 2),
                label="No_mesh",
            )
        )
    else:  # Symmetry
        # Bottom line
        Z1 = Rgap_mag_int
        Z2 = Rgap_mec_int + W_sb
        Z3 = Z2 * exp(1j * 2 * pi / sym)
        Z4 = Z1 * exp(1j * 2 * pi / sym)
        airgap_lines = list()
        airgap_lines.append(Segment(begin=Z1, end=Z2, label="airgap_line_1"))
        airgap_lines.append(
            Arc1(begin=Z2, end=Z3, radius=Rgap_mec_int + W_sb, label="sliding_line")
        )
        airgap_lines.append(Segment(begin=Z3, end=Z4, label="airgap_line_1"))
        surf_list.append(
            SurfLine(
                line_list=airgap_lines,
                point_ref=(Z2 - W_sb / 2) * exp(1j * pi / sym),
                label="Airgap",
            )
        )
        # Top line
        Z5 = Rgap_mag_ext
        Z6 = Rgap_mec_ext - W_sb
        Z7 = Z6 * exp(1j * 2 * pi / sym)
        Z8 = Z5 * exp(1j * 2 * pi / sym)
        airgap_lines = list()
        airgap_lines.append(Segment(begin=Z5, end=Z6, label="airgap_line_2"))
        airgap_lines.append(
            Arc1(begin=Z6, end=Z7, radius=Rgap_mec_ext - W_sb, label="sliding_line")
        )
        airgap_lines.append(Segment(begin=Z7, end=Z8, label="airgap_line_2"))
        surf_list.append(
            SurfLine(
                line_list=airgap_lines,
                point_ref=(Z6 + W_sb / 2) * exp(1j * pi / sym),
                label="Airgap",
            )
        )

    return surf_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/Frame.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/Frame
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.Frame.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error

try:
    from ..Methods.Machine.Frame.comp_height_eq import comp_height_eq
except ImportError as error:
    comp_height_eq = error

try:
    from ..Methods.Machine.Frame.comp_mass import comp_mass
except ImportError as error:
    comp_mass = error

try:
    from ..Methods.Machine.Frame.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Machine.Frame.comp_volume import comp_volume
except ImportError as error:
    comp_volume = error

try:
    from ..Methods.Machine.Frame.get_length import get_length
except ImportError as error:
    get_length = error

try:
    from ..Methods.Machine.Frame.plot import plot
except ImportError as error:
    plot = error


from ._check import InitUnKnowClassError
from .Material import Material


class Frame(FrozenClass):
    """machine frame"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.Frame.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Frame method build_geometry: " + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # cf Methods.Machine.Frame.comp_height_eq
    if isinstance(comp_height_eq, ImportError):
        comp_height_eq = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Frame method comp_height_eq: " + str(comp_height_eq)
                )
            )
        )
    else:
        comp_height_eq = comp_height_eq
    # cf Methods.Machine.Frame.comp_mass
    if isinstance(comp_mass, ImportError):
        comp_mass = property(
            fget=lambda x: raise_(
                ImportError("Can't use Frame method comp_mass: " + str(comp_mass))
            )
        )
    else:
        comp_mass = comp_mass
    # cf Methods.Machine.Frame.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError("Can't use Frame method comp_surface: " + str(comp_surface))
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Machine.Frame.comp_volume
    if isinstance(comp_volume, ImportError):
        comp_volume = property(
            fget=lambda x: raise_(
                ImportError("Can't use Frame method comp_volume: " + str(comp_volume))
            )
        )
    else:
        comp_volume = comp_volume
    # cf Methods.Machine.Frame.get_length
    if isinstance(get_length, ImportError):
        get_length = property(
            fget=lambda x: raise_(
                ImportError("Can't use Frame method get_length: " + str(get_length))
            )
        )
    else:
        get_length = get_length
    # cf Methods.Machine.Frame.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use Frame method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, Lfra=0.35, Rint=0.2, Rext=0.2, mat_type=-1, init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Lfra" in list(init_dict.keys()):
                Lfra = init_dict["Lfra"]
            if "Rint" in list(init_dict.keys()):
                Rint = init_dict["Rint"]
            if "Rext" in list(init_dict.keys()):
                Rext = init_dict["Rext"]
            if "mat_type" in list(init_dict.keys()):
                mat_type = init_dict["mat_type"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.Lfra = Lfra
        self.Rint = Rint
        self.Rext = Rext
        self.mat_type = mat_type

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Frame_str = ""
        if self.parent is None:
            Frame_str += "parent = None " + linesep
        else:
            Frame_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Frame_str += "Lfra = " + str(self.Lfra) + linesep
        Frame_str += "Rint = " + str(self.Rint) + linesep
        Frame_str += "Rext = " + str(self.Rext) + linesep
        if self.mat_type is not None:
            tmp = self.mat_type.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Frame_str += "mat_type = " + tmp
        else:
            Frame_str += "mat_type = None" + linesep + linesep
        return Frame_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.Lfra != self.Lfra:
            return False
        if other.Rint != self.Rint:
            return False
        if other.Rext != self.Rext:
            return False
        if other.mat_type != self.mat_type:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._Lfra != self._Lfra:
            diff_list.append(name + ".Lfra")
        if other._Rint != self._Rint:
            diff_list.append(name + ".Rint")
        if other._Rext != self._Rext:
            diff_list.append(name + ".Rext")
        if (other.mat_type is None and self.mat_type is not None) or (
            other.mat_type is not None and self.mat_type is None
        ):
            diff_list.append(name + ".mat_type None mismatch")
        elif self.mat_type is not None:
            diff_list.extend(
                self.mat_type.compare(other.mat_type, name=name + ".mat_type")
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.Lfra)
        S += getsizeof(self.Rint)
        S += getsizeof(self.Rext)
        S += getsizeof(self.mat_type)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Frame_dict = dict()
        Frame_dict["Lfra"] = self.Lfra
        Frame_dict["Rint"] = self.Rint
        Frame_dict["Rext"] = self.Rext
        if self.mat_type is None:
            Frame_dict["mat_type"] = None
        else:
            Frame_dict["mat_type"] = self.mat_type.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        Frame_dict["__class__"] = "Frame"
        return Frame_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Lfra = None
        self.Rint = None
        self.Rext = None
        if self.mat_type is not None:
            self.mat_type._set_None()

    def _get_Lfra(self):
        """getter of Lfra"""
        return self._Lfra

    def _set_Lfra(self, value):
        """setter of Lfra"""
        check_var("Lfra", value, "float", Vmin=0)
        self._Lfra = value

    Lfra = property(
        fget=_get_Lfra,
        fset=_set_Lfra,
        doc=u"""frame length [m]

        :Type: float
        :min: 0
        """,
    )

    def _get_Rint(self):
        """getter of Rint"""
        return self._Rint

    def _set_Rint(self, value):
        """setter of Rint"""
        check_var("Rint", value, "float", Vmin=0)
        self._Rint = value

    Rint = property(
        fget=_get_Rint,
        fset=_set_Rint,
        doc=u"""frame internal radius

        :Type: float
        :min: 0
        """,
    )

    def _get_Rext(self):
        """getter of Rext"""
        return self._Rext

    def _set_Rext(self, value):
        """setter of Rext"""
        check_var("Rext", value, "float", Vmin=0)
        self._Rext = value

    Rext = property(
        fget=_get_Rext,
        fset=_set_Rext,
        doc=u"""Frame external radius

        :Type: float
        :min: 0
        """,
    )

    def _get_mat_type(self):
        """getter of mat_type"""
        return self._mat_type

    def _set_mat_type(self, value):
        """setter of mat_type"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "mat_type"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Material()
        check_var("mat_type", value, "Material")
        self._mat_type = value

        if self._mat_type is not None:
            self._mat_type.parent = self

    mat_type = property(
        fget=_get_mat_type,
        fset=_set_mat_type,
        doc=u"""Frame material

        :Type: Material
        """,
    )



# -*- coding: utf-8 -*-

from ....Methods.Simulation.Input import InputError
from ....Classes.LossModel import LossModel
from ....Classes.MeshSolution import MeshSolution
from SciDataTool import DataND


# TODO case of 'part' string may be problematic -> check
#      should e.g. be 'Stator' but FEMM groups are lower case
# TODO maybe implement machine.get_part_by_label and use here
#      or input label to loss model, so the model has to take care
def run(self):
    """Run the Loss module"""
    if self.parent is None:
        raise InputError("ERROR: The Loss object must be in a Simulation object to run")
    if self.parent.parent is None:
        raise InputError("ERROR: The Loss object must be in an Output object to run")

    # get logger
    logger = self.get_logger()

    # get output
    output = self.parent.parent

    # replicate model_list for solutions with respective object types
    output.loss.meshsol_list = [None for ii in self.model_list]
    output.loss.loss_list = [None for ii in self.model_list]

    # iterate through the loss models and compute losses
    for part_label in self.model_index.keys():
        if part_label not in output.loss.loss_index.keys():
            output.loss.loss_index[part_label] = {}
        for key, index in self.model_index[part_label].items():
            if key not in output.loss.loss_index[part_label].keys():
                output.loss.loss_index[part_label][key] = {}
            # run the loss model
            output.loss.loss_index[part_label][key] = index
            data, mshsol = self.model_list[index].comp_loss(output, part_label)
            # store the results
            if data is not None:
                output.loss.loss_list[index] = data
            if mshsol is not None:
                output.loss.meshsol_list[index] = mshsol



# -*- coding: utf-8 -*-

from numpy import angle


def comp_angle_opening(self):
    """Compute the average opening angle of the Slot

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    line_list = self.build_geometry()
    Z1 = line_list[0].get_begin()
    Z2 = line_list[-1].get_end()

    return angle(Z2) - angle(Z1)



# -*- coding: utf-8 -*-

from numpy import exp, pi, tan


def comp_surface(self):
    """Compute the surface of the Hole

    Parameters
    ----------
    self : HoleM52
        A HoleM52 object

    Returns
    -------
    S: float
        Surface of the Hole. [m**2]

    """

    Rext = self.get_Rext()

    alpha = self.comp_alpha()

    # Angle between (P1,P2) and (0,P0) is slot_pitch /2
    # It is also the angle (P1,P2,S)
    hsp = pi / self.Zh  # Half Slot Pitch

    # Distance P1,P9
    D19 = ((Rext - self.H0) * exp(1j * alpha / 2)).imag * 2

    # S is the intersectioni between (P1,P9) and the parallel to x passing by P2
    D1S = tan(hsp) * (self.H1 - self.H2)

    W1 = D19 / 2.0 - D1S - self.W0 / 2.0

    Smag = self.comp_surface_magnets()
    # Two air trapeze area
    Strap = ((D19 / 2.0 - self.W0 / 2.0) + W1) * (self.H1 - self.H2)

    return Smag + Strap



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surfaces(self):
    """Compute the Lamination surface (Total, Vent).

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    S_dict: dict
        Lamination surface dictionnary (Slam, Svent, Syoke, Steeth, Sslot) [m**2]

    """

    # Surface of the external disk
    S_ext = (self.Rext ** 2) * pi
    # Surface of the internal disk
    S_int = (self.Rint ** 2) * pi
    # Surface of lamination without hole
    Slam = S_ext - S_int

    # Surface of the ventilation ducts on the yoke
    Svent = self.comp_surface_axial_vent()

    return {
        "Slam": Slam - Svent,
        "Svent": Svent,
        "Syoke": Slam,
        "Steeth": 0,
        "Sslot": 0,
    }



# -*- coding: utf-8 -*-
from numpy import pi


def comp_surface(self):
    """Compute the complete surface of the Hole

    Parameters
    ----------
    self : HoleM54
        A HoleM54 object

    Returns
    -------
    S: float
        Surface of the Hole [m**2]

    """

    Sint = pi * self.R1 ** 2
    Sext = pi * (self.R1 + self.H1) ** 2
    Sarc = (Sext - Sint) * (self.W0 / (2 * pi))

    return Sarc + pi * (self.H1 / 2) ** 2



# -*- coding: utf-8 -*-

from .plot_2D import plot_2D
from . import unit_dict, norm_dict, axes_dict
from ...definitions import config_dict
from numpy import squeeze, split, array, where, max as np_max


def plot_2D_Data(
    data,
    *arg_list,
    is_norm=False,
    unit="SI",
    data_list=[],
    legend_list=[],
    color_list=[],
    save_path=None,
    x_min=None,
    x_max=None,
    y_min=None,
    y_max=None,
    is_logscale_x=False,
    is_logscale_y=False,
    is_disp_title=True,
    is_grid=True,
    is_auto_ticks=True,
    fig=None,
    ax=None,
    barwidth=100,
    type_plot=None,
    fund_harm_dict=None,
    is_show_fig=None,
    win_title=None,
):
    """Plots a field as a function of time

    Parameters
    ----------
    data : Data
        a Data object
    *arg_list : list of str
        arguments to specify which axes to plot
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    data_list : list
        list of Data objects to compare
    legend_list : list
        list of legends to use for each Data object (including reference one) instead of data.name
    color_list : list
        list of colors to use for each Data object
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    x_min : float
        minimum value for the x-axis
    x_max : float
        maximum value for the x-axis
    y_min : float
        minimum value for the y-axis
    y_max : float
        maximum value for the y-axis
    is_logscale_x : bool
        boolean indicating if the x-axis must be set in logarithmic scale
    is_logscale_y : bool
        boolean indicating if the y-axis must be set in logarithmic scale
    is_disp_title : bool
        boolean indicating if the title must be displayed
    is_grid : bool
        boolean indicating if the grid must be displayed
    is_auto_ticks : bool
        in fft, adjust ticks to freqs (deactivate if too close)
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        ax on which to plot the data
    barwidth : float
        barwidth scaling factor, only if type_plot = "bargraph"
    type_plot : str
        type of 2D graph : "curve", "bargraph", "barchart" or "quiver"
    fund_harm_dict : dict
        Dict containing axis name as key and frequency/order/wavenumber of fundamental harmonic as value to display fundamental harmonic in red in the fft
    is_show_fig : bool
        True to show figure after plot
    win_title : str
        Title of the plot window
    """
    # Extract arg_list it the function called from another script with *arg_list
    if len(arg_list) == 1 and type(arg_list[0]) == tuple:
        arg_list = arg_list[0]

    # Get colors and line_styles from config_dict
    curve_colors = config_dict["PLOT"]["COLOR_DICT"]["CURVE_COLORS"]
    phase_colors = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"]
    line_styles = config_dict["PLOT"]["LINE_STYLE"]

    # Set unit
    if unit == "SI":
        unit = data.unit

    # Detect if is fft, build first title part and ylabel
    is_fft = False
    if any("wavenumber" in s for s in arg_list) or any("freqs" in s for s in arg_list):
        is_fft = True
        if "dB" in unit:
            unit_str = (
                "[" + unit + " re. " + str(data.normalizations["ref"]) + data.unit + "]"
            )
        else:
            unit_str = "[" + unit + "]"
        if data.symbol == "Magnitude":
            ylabel = "Magnitude " + unit_str
        else:
            ylabel = r"$|\widehat{" + data.symbol + "}|$ " + unit_str
        if data_list == []:
            title1 = "FFT of " + data.name.lower() + " "
        else:
            title1 = "Comparison of " + data.name.lower() + " FFT "
    else:
        if data_list == []:
            # title = data.name.capitalize() + " for " + ', '.join(arg_list_str)
            title1 = data.name.capitalize() + " "
        else:
            # title = data.name.capitalize() + " for " + ', '.join(arg_list_str)
            title1 = "Comparison of " + data.name.lower() + " "
        if is_norm:
            ylabel = (
                r"$\frac{" + data.symbol + "}{" + data.symbol + "_0}\, [" + unit + "]$"
            )
        else:
            ylabel = r"$" + data.symbol + "\, [" + unit + "]$"

    # Extract field and axes
    Xdatas = []
    Ydatas = []
    data_list2 = [data] + data_list
    for i, d in enumerate(data_list2):
        if is_fft:
            result = d.get_magnitude_along(arg_list, unit=unit, is_norm=is_norm)
            if i == 0:
                axes_list = result.pop("axes_list")
                axes_dict_other = result.pop("axes_dict_other")
                result_0 = result
            Ydatas.append(result.pop(d.symbol))
            Xdatas.append(result[list(result)[0]])
        else:
            result = d.get_along(arg_list, unit=unit, is_norm=is_norm)
            if i == 0:
                axes_list = result.pop("axes_list")
                axes_dict_other = result.pop("axes_dict_other")
                result_0 = result
            Ydatas.append(result.pop(d.symbol))
            Xdatas.append(result[list(result)[0]])

    # Find main axis as the axis with the most values
    # main_axis = axes_list[0]  # max(axes_list, key=lambda x: x.values.size)

    # Build xlabel and title parts 2 and 3
    title2 = ""
    title3 = " for "
    for axis in axes_list:
        # Title part 2
        if axis.extension in [
            "whole",
            "interval",
            "oneperiod",
            "antiperiod",
            "smallestperiod",
        ]:
            if axis.name in axes_dict:
                name = axes_dict[axis.name]
            else:
                name = axis.name
            title2 = "over " + axis.name.lower()
            if axis.unit == "SI":
                unit = unit_dict[axis.name]
                xlabel = name.capitalize() + " [" + unit + "]"
                main_axis_name = name
            elif axis.unit in norm_dict:
                xlabel = norm_dict[axis.unit]
                main_axis_name = axis.unit
            else:
                unit = axis.unit
                xlabel = name.capitalize() + " [" + unit + "]"
                main_axis_name = name
            if (
                axis.name == "angle"
                and axis.unit == "°"
                and round(np_max(axis.values) / 6) % 5 == 0
            ):
                xticks = [i * round(np_max(axis.values) / 6) for i in range(7)]
            else:
                xticks = None
        else:
            if axis.unit == "SI":
                unit = unit_dict[axis.name]
            elif axis.unit in norm_dict:
                unit = norm_dict[axis.unit]
            else:
                unit = axis.unit

            # if result[axis.name].size >1:
            #     axis_str = result[axis.name].astype(
            #     str
            # )
            #     axis_str = "[" + ",".join(axis_str) +"]"
            # else:
            #     axis_str = str(result[axis.name][0])
            axis_str = str(
                result_0[axis.name]
            )  # TODO: smart conversion of float to str

            title3 += axis.name + "=" + axis_str + " " + unit + ", "

    # Title part 4 containing axes that are here but not involved in requested axes
    title4 = ""
    for axis_name in axes_dict_other:
        title4 += (
            axis_name
            + "="
            + str(axes_dict_other[axis_name][0])
            + " "
            + axes_dict_other[axis_name][1]
            + ", "
        )

    if title3 == " for " and title4 == "":
        title3 = ""

    # Concatenate all title parts
    title = title1 + title2 + title3 + title4

    # Remove last coma due to title3 or title4
    title = title.rstrip(", ")

    # Detect how many curves are overlaid, build legend and color lists
    if legend_list == [] and data_list != []:
        legend_list = ["[" + d.name + "] " for d in data_list2]
    elif legend_list == []:
        legend_list = ["" for d in data_list2]
    else:
        legend_list = ["[" + leg + "] " for leg in legend_list]
    legends = []
    colors = []
    linestyle_list = []
    for i, d in enumerate(data_list2):
        is_overlay = False
        for axis in axes_list:
            if axis.extension == "list":
                is_overlay = True
                n_curves = len(axis.values)
                if axis.unit == "SI":
                    unit = unit_dict[axis.name]
                elif axis.unit in norm_dict:
                    unit = norm_dict[axis.unit]
                else:
                    unit = axis.unit
                legends += [
                    legend_list[i]
                    + axis.name
                    + "= "
                    + str(axis.values.tolist()[j])
                    + " "
                    + unit
                    for j in range(n_curves)
                ]
                colors += [
                    phase_colors[(i * n_curves + j) % len(phase_colors)]
                    for j in range(n_curves)
                ]
                linestyle_list += [line_styles[i] for j in range(n_curves)]

        if not is_overlay:
            legends += [legend_list[i]]
            colors += [curve_colors[i]]
            linestyle_list += [line_styles[i]]

    # Set colors_list to colors that has just been built
    if color_list == []:
        color_list = colors

    # Split Ydatas if the plot overlays several curves
    if is_overlay:
        Ydata = []
        for d in Ydatas:
            if d.ndim != 1:
                axis_index = where(array(d.shape) == n_curves)[0]
                if axis_index.size > 1:
                    print("WARNING, several axes with same dimensions")
                Ydata += split(d, n_curves, axis=axis_index[0])
            else:
                Ydata += [d]
        Ydatas = [squeeze(d) for d in Ydata]
        Xdata = []
        for i in range(len(data_list2)):
            Xdata += [Xdatas[i] for x in range(n_curves)]
        Xdatas = Xdata

    # Call generic plot function
    if is_fft:
        freqs = Xdatas[0]
        if is_auto_ticks:
            indices = [
                ind for ind, y in enumerate(Ydatas[0]) if abs(y) > abs(0.01 * np_max(y))
            ]
            xticks = freqs[indices]
        else:
            xticks = None

        # Force bargraph for fft if type_graph not specified
        if type_plot is None:
            type_plot = "bargraph"

        # Option to draw fundamental harmonic in red
        if fund_harm_dict is None:
            fund_harm = None
        else:
            # Activate the option only if main axis is in dict and only one Data is plotted
            if main_axis_name in fund_harm_dict and len(Ydatas) == 1:
                fund_harm = fund_harm_dict[main_axis_name]
            else:
                # Deactivate the option
                fund_harm = None

        plot_2D(
            Xdatas,
            Ydatas,
            legend_list=legends,
            color_list=color_list,
            fig=fig,
            ax=ax,
            title=title,
            xlabel=xlabel,
            ylabel=ylabel,
            type_plot=type_plot,
            x_min=x_min,
            x_max=x_max,
            y_min=y_min,
            y_max=y_max,
            is_logscale_x=is_logscale_x,
            is_logscale_y=is_logscale_y,
            is_disp_title=is_disp_title,
            is_grid=is_grid,
            xticks=xticks,
            save_path=save_path,
            barwidth=barwidth,
            fund_harm=fund_harm,
            is_show_fig=is_show_fig,
            win_title=win_title,
        )

    else:

        # Force curve plot if type_plot not specified
        if type_plot is None:
            type_plot = "curve"

        plot_2D(
            Xdatas,
            Ydatas,
            legend_list=legends,
            color_list=color_list,
            fig=fig,
            ax=ax,
            title=title,
            xlabel=xlabel,
            ylabel=ylabel,
            type_plot=type_plot,
            x_min=x_min,
            x_max=x_max,
            y_min=y_min,
            y_max=y_max,
            is_logscale_x=is_logscale_x,
            is_logscale_y=is_logscale_y,
            is_disp_title=is_disp_title,
            is_grid=is_grid,
            xticks=xticks,
            linestyle_list=linestyle_list,
            save_path=save_path,
            is_show_fig=is_show_fig,
            win_title=win_title,
        )



# -*- coding: utf-8 -*-

from PySide2.QtCore import Qt, Signal
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QMessageBox, QWidget

from .....Classes.Winding import Winding
from .....Classes.WindingCW1L import WindingCW1L
from .....Classes.WindingCW2LR import WindingCW2LR
from .....Classes.WindingCW2LT import WindingCW2LT
from .....Classes.WindingDW1L import WindingDW1L
from .....Classes.WindingDW2L import WindingDW2L
from .....Functions.Winding.comp_wind_periodicity import comp_wind_periodicity
from .....GUI.Dialog.DMachineSetup.SWindPat.Gen_SWindPat import Gen_SWindPat
from .....Methods.Machine.Winding import WindingError

# For the Pattern combobox
TYPE_INDEX = [WindingCW2LT, WindingCW1L, WindingDW2L, WindingDW1L, WindingCW2LR]


class SWindPat(Gen_SWindPat, QWidget):
    """Step to define the winding pattern"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for DMachineSetup nav
    step_name = "Winding (1)"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the GUI according to machine

        Parameters
        ----------
        self : SWindPat
            A SWindPat widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Set Help URL
        self.b_help.url = "https://pyleecan.org/winding.convention.html"

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        # Fill the fields with the machine values (if they're filled)
        if self.is_stator:
            self.obj = machine.stator
        else:
            self.obj = machine.rotor

        if machine.type_machine == 9 and not self.is_stator:
            # Enforce tooth winding for WRSM rotor
            self.obj.winding = WindingCW2LT(init_dict=self.obj.winding.as_dict())
            self.obj.winding.qs = 1
            self.b_preview.setEnabled(False)
            self.si_qs.setEnabled(False)
            self.c_wind_type.setEnabled(False)
            self.c_wind_type.setCurrentIndex(0)
            self.c_wind_type.setItemText(0, "DC wound winding for salient pole")
        else:
            self.b_preview.setEnabled(True)
            self.si_qs.setEnabled(True)
            self.c_wind_type.setEnabled(True)
            self.c_wind_type.setItemText(0, "Double Layer Concentrated Orthoradial")

        # Set the current Winding pattern
        if self.obj.winding is None or type(self.obj.winding) is Winding:
            # The default type_winding is WindingCW2LT
            if type(self.obj.winding) is Winding:
                self.obj.winding = WindingCW2LT(init_dict=self.obj.winding.as_dict())
            else:
                self.obj.winding = WindingCW2LT()
            self.c_wind_type.setCurrentIndex(0)
        else:
            self.c_wind_type.setCurrentIndex(TYPE_INDEX.index(type(self.obj.winding)))
        self.update_graph()

        if type(self.obj.winding) is WindingDW2L:
            if self.obj.winding.coil_pitch is None:
                self.obj.winding.coil_pitch = 0
            self.si_coil_pitch.setValue(self.obj.winding.coil_pitch)

        if self.obj.winding.Nslot_shift_wind is None:
            self.obj.winding.Nslot_shift_wind = 0
        self.si_Nslot.setValue(self.obj.winding.Nslot_shift_wind)

        if self.obj.winding.qs is None:  # default value
            self.obj.winding.qs = 3
        self.si_qs.setValue(self.obj.winding.qs)

        if self.obj.winding.Ntcoil is None:
            self.obj.winding.Ntcoil = 1  # Default value for preview

        if self.obj.winding.is_reverse_wind is None:
            self.obj.winding.is_reverse_wind = False
        if self.obj.winding.is_reverse_wind:
            self.is_reverse.setCheckState(Qt.Checked)
        else:
            self.is_reverse.setCheckState(Qt.Unchecked)

        # Display shape of wind_mat
        self.set_output()
        self.hide_coil_pitch()

        # Connect the signal/slot
        self.c_wind_type.currentIndexChanged.connect(self.set_type)
        self.si_qs.editingFinished.connect(self.set_qs)
        self.si_coil_pitch.editingFinished.connect(self.set_coil_pitch)
        self.si_Nslot.valueChanged.connect(self.set_Nslot)
        self.is_reverse.stateChanged.connect(self.set_is_reverse_wind)
        self.b_preview.clicked.connect(self.s_plot)

    def set_type(self, index):
        """Signal to update the winding type

        Parameters
        ----------
        self : SWindPat
            A SWindPat object
        index : int
            Index of selected type
        """

        w_dict = Winding.as_dict(self.obj.winding)
        self.obj.winding = TYPE_INDEX[index](init_dict=w_dict)

        # Update out_shape
        self.set_output()
        self.hide_coil_pitch()
        # Update image
        self.update_graph()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_qs(self):
        """Signal to update the value of qs according to the spinbox

        Parameters
        ----------
        self : SWindPat
            A SWindPat object
        """
        self.obj.winding.qs = self.si_qs.value()
        self.set_output()
        self.update_graph()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_coil_pitch(self):
        """Signal to update the value of coil_pitch according to the spinbox

        Parameters
        ----------
        self : SWindPat
            A SWindPat object
        """
        self.obj.winding.coil_pitch = self.si_coil_pitch.value()
        self.set_output()
        self.update_graph()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Nslot(self):
        """Signal to update the value of Nslot_shift_wind according to the
        spinbox

        Parameters
        ----------
        self : SWindPat
            A SWindPat object
        """
        self.obj.winding.Nslot_shift_wind = self.si_Nslot.value()
        self.update_graph()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_is_reverse_wind(self, value):
        """Signal to update the value of is_reverse_wind according to the
        widget

        Parameters
        ----------
        self : SWindPat
            A SWindPat object
        value :
            New value of is_reverse_wind
        """

        value = self.is_reverse.isChecked()
        self.update_graph()
        self.obj.winding.is_reverse_wind = value
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def hide_coil_pitch(self):
        """Show coil_pitch only if type(winding) is WindingDW2L

        Parameters
        ----------
        self : SWindPat
            A SWindPat object
        """
        self.si_coil_pitch.blockSignals(True)
        if type(self.obj.winding) is WindingDW2L:
            self.si_coil_pitch.show()
            self.in_coil_pitch.show()
        else:
            self.si_coil_pitch.hide()
            self.in_coil_pitch.hide()
        self.si_coil_pitch.blockSignals(False)

    def set_output(self):
        """Update the shape and period Label to match the current winding setup

        Parameters
        ----------
        self : SWindPat
            a SWindPat object
        """

        wind = self.obj.winding  # For readability
        # Wind_matrix is a matrix of dimension [Nlay_rad, Nlay_tan, Zs, qs]
        # Nlay_rad and Nlay_tan depend of the winding type
        (Nrad, Ntan) = wind.get_dim_wind()
        Nlay = str(Nrad) + ", " + str(Ntan) + ", "

        # Zs should be set, but to be sure:
        if self.obj.slot.Zs is None:
            Zs = "?, "
        else:
            Zs = str(self.obj.slot.Zs) + ", "

        if wind.qs is None:
            qs = "?]"
        else:
            qs = str(wind.qs) + "]"

        self.out_shape.setText(self.tr("Matrix shape [") + Nlay + Zs + qs)

        try:
            ms = str(self.obj.slot.Zs / (wind.p * wind.qs * 2.0))
        except TypeError:  # One of the value is None
            ms = "?"
        if self.obj.is_stator:
            self.out_ms.setText(self.tr("ms = Zs / (2*p*qs) = ") + ms)
        else:
            self.out_ms.setText(self.tr("ms = Zr / (2*p*qr) = ") + ms)

        try:
            wind_mat = wind.comp_connection_mat(self.obj.slot.Zs)
            Nperw = str(comp_wind_periodicity(wind_mat)[0])
        except Exception:  # Unable to compution the connection matrix
            Nperw = "?"

        self.out_Nperw.setText(self.tr("Nperw: ") + Nperw)

    def update_graph(self):
        """Plot the lamination with/without the winding"""
        # Plot the lamination in the viewer fig
        try:
            self.obj.plot(fig=self.w_viewer.fig, is_show_fig=False)
        except:
            pass

        # Update the Graph
        self.w_viewer.axes.set_axis_off()
        self.w_viewer.axes.axis("equal")
        if self.w_viewer.axes.get_legend() is not None:
            self.w_viewer.axes.get_legend().remove()
        self.w_viewer.draw()

    def s_plot(self):
        """Plot a preview of the winding in a popup

        Parameters
        ----------
        self : SWindPat
            A SWindPat object
        """
        try:
            self.obj.plot_winding()
        except (AssertionError, WindingError) as e:
            QMessageBox().critical(self, self.tr("Error"), str(e))

    @staticmethod
    def check(lamination):
        """Check that the lamination have all the needed field set

        Parameters
        ----------
        lamination : Lamination
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """
        try:
            # Check that everything is set
            if lamination.winding.qs is None:
                return "You must set qs !"
            if lamination.winding.Nslot_shift_wind is None:
                lamination.winding.Nslot_shift_wind = 0
            if lamination.winding.is_reverse_wind is None:
                lamination.winding.is_reverse_wind = False
        except Exception as e:
            return str(e)



# -*- coding: utf-8 -*-
from matplotlib.patches import Polygon
from ....definitions import config_dict

PATCH_EDGE = config_dict["PLOT"]["COLOR_DICT"]["PATCH_EDGE"]
PATCH_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PATCH_COLOR"]
PATCH_COLOR_ALPHA = config_dict["PLOT"]["COLOR_DICT"]["PATCH_COLOR_ALPHA"]
PATCH_EDGE_ALPHA = config_dict["PLOT"]["COLOR_DICT"]["PATCH_EDGE_ALPHA"]


def get_patches(
    self, color=PATCH_COLOR, edgecolor=PATCH_EDGE, is_edge_only=False, linestyle=None
):
    """Returns the PolarArc Patch to be display in matplotlib

    Parameters
    ----------
    self : SurfLine
        a SurfLine object
    color :
        the color of the patch (Default value = PATCH_COLOR)
    edgecolor :
        the edge color of the patch (Default value = PATCH_EDGE)
    is_edge_only: bool
        To set the transparancy of the face color to 0 and 1 for the edge color
    linestyle : str
        Line style of the edge {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}

    Returns
    -------
    patch_list : list of matplotlib.patches.Polygon
        List of patches corresponding to the surface

    """
    patch_list = self.out_surf.get_patches(
        color=color, edgecolor=edgecolor, is_edge_only=is_edge_only, linestyle=linestyle
    )
    # No color for inner surface
    patch_list.extend(
        self.in_surf.get_patches(is_edge_only=is_edge_only, linestyle=linestyle)
    )
    return patch_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Slot/SlotW25.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Slot/SlotW25
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Slot import Slot

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Slot.SlotW25._comp_point_coordinate import _comp_point_coordinate
except ImportError as error:
    _comp_point_coordinate = error

try:
    from ..Methods.Slot.SlotW25.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error

try:
    from ..Methods.Slot.SlotW25.build_geometry_active import build_geometry_active
except ImportError as error:
    build_geometry_active = error

try:
    from ..Methods.Slot.SlotW25.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Slot.SlotW25.comp_angle_opening import comp_angle_opening
except ImportError as error:
    comp_angle_opening = error

try:
    from ..Methods.Slot.SlotW25.comp_height import comp_height
except ImportError as error:
    comp_height = error

try:
    from ..Methods.Slot.SlotW25.comp_height_active import comp_height_active
except ImportError as error:
    comp_height_active = error

try:
    from ..Methods.Slot.SlotW25.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Slot.SlotW25.comp_surface_active import comp_surface_active
except ImportError as error:
    comp_surface_active = error

try:
    from ..Methods.Slot.SlotW25.get_surface_active import get_surface_active
except ImportError as error:
    get_surface_active = error

try:
    from ..Methods.Slot.SlotW25.plot_schematics import plot_schematics
except ImportError as error:
    plot_schematics = error


from ._check import InitUnKnowClassError


class SlotW25(Slot):

    VERSION = 1
    IS_SYMMETRICAL = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Slot.SlotW25._comp_point_coordinate
    if isinstance(_comp_point_coordinate, ImportError):
        _comp_point_coordinate = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method _comp_point_coordinate: "
                    + str(_comp_point_coordinate)
                )
            )
        )
    else:
        _comp_point_coordinate = _comp_point_coordinate
    # cf Methods.Slot.SlotW25.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method build_geometry: " + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # cf Methods.Slot.SlotW25.build_geometry_active
    if isinstance(build_geometry_active, ImportError):
        build_geometry_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method build_geometry_active: "
                    + str(build_geometry_active)
                )
            )
        )
    else:
        build_geometry_active = build_geometry_active
    # cf Methods.Slot.SlotW25.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use SlotW25 method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Slot.SlotW25.comp_angle_opening
    if isinstance(comp_angle_opening, ImportError):
        comp_angle_opening = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method comp_angle_opening: "
                    + str(comp_angle_opening)
                )
            )
        )
    else:
        comp_angle_opening = comp_angle_opening
    # cf Methods.Slot.SlotW25.comp_height
    if isinstance(comp_height, ImportError):
        comp_height = property(
            fget=lambda x: raise_(
                ImportError("Can't use SlotW25 method comp_height: " + str(comp_height))
            )
        )
    else:
        comp_height = comp_height
    # cf Methods.Slot.SlotW25.comp_height_active
    if isinstance(comp_height_active, ImportError):
        comp_height_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method comp_height_active: "
                    + str(comp_height_active)
                )
            )
        )
    else:
        comp_height_active = comp_height_active
    # cf Methods.Slot.SlotW25.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method comp_surface: " + str(comp_surface)
                )
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Slot.SlotW25.comp_surface_active
    if isinstance(comp_surface_active, ImportError):
        comp_surface_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method comp_surface_active: "
                    + str(comp_surface_active)
                )
            )
        )
    else:
        comp_surface_active = comp_surface_active
    # cf Methods.Slot.SlotW25.get_surface_active
    if isinstance(get_surface_active, ImportError):
        get_surface_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method get_surface_active: "
                    + str(get_surface_active)
                )
            )
        )
    else:
        get_surface_active = get_surface_active
    # cf Methods.Slot.SlotW25.plot_schematics
    if isinstance(plot_schematics, ImportError):
        plot_schematics = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotW25 method plot_schematics: " + str(plot_schematics)
                )
            )
        )
    else:
        plot_schematics = plot_schematics
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        W3=0.003,
        H2=0.003,
        W4=0.003,
        H1=0.003,
        Zs=36,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "W3" in list(init_dict.keys()):
                W3 = init_dict["W3"]
            if "H2" in list(init_dict.keys()):
                H2 = init_dict["H2"]
            if "W4" in list(init_dict.keys()):
                W4 = init_dict["W4"]
            if "H1" in list(init_dict.keys()):
                H1 = init_dict["H1"]
            if "Zs" in list(init_dict.keys()):
                Zs = init_dict["Zs"]
        # Set the properties (value check and convertion are done in setter)
        self.W3 = W3
        self.H2 = H2
        self.W4 = W4
        self.H1 = H1
        # Call Slot init
        super(SlotW25, self).__init__(Zs=Zs)
        # The class is frozen (in Slot init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        SlotW25_str = ""
        # Get the properties inherited from Slot
        SlotW25_str += super(SlotW25, self).__str__()
        SlotW25_str += "W3 = " + str(self.W3) + linesep
        SlotW25_str += "H2 = " + str(self.H2) + linesep
        SlotW25_str += "W4 = " + str(self.W4) + linesep
        SlotW25_str += "H1 = " + str(self.H1) + linesep
        return SlotW25_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Slot
        if not super(SlotW25, self).__eq__(other):
            return False
        if other.W3 != self.W3:
            return False
        if other.H2 != self.H2:
            return False
        if other.W4 != self.W4:
            return False
        if other.H1 != self.H1:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Slot
        diff_list.extend(super(SlotW25, self).compare(other, name=name))
        if other._W3 != self._W3:
            diff_list.append(name + ".W3")
        if other._H2 != self._H2:
            diff_list.append(name + ".H2")
        if other._W4 != self._W4:
            diff_list.append(name + ".W4")
        if other._H1 != self._H1:
            diff_list.append(name + ".H1")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Slot
        S += super(SlotW25, self).__sizeof__()
        S += getsizeof(self.W3)
        S += getsizeof(self.H2)
        S += getsizeof(self.W4)
        S += getsizeof(self.H1)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Slot
        SlotW25_dict = super(SlotW25, self).as_dict(**kwargs)
        SlotW25_dict["W3"] = self.W3
        SlotW25_dict["H2"] = self.H2
        SlotW25_dict["W4"] = self.W4
        SlotW25_dict["H1"] = self.H1
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        SlotW25_dict["__class__"] = "SlotW25"
        return SlotW25_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.W3 = None
        self.H2 = None
        self.W4 = None
        self.H1 = None
        # Set to None the properties inherited from Slot
        super(SlotW25, self)._set_None()

    def _get_W3(self):
        """getter of W3"""
        return self._W3

    def _set_W3(self, value):
        """setter of W3"""
        check_var("W3", value, "float", Vmin=0)
        self._W3 = value

    W3 = property(
        fget=_get_W3,
        fset=_set_W3,
        doc=u"""Teeth bottom width

        :Type: float
        :min: 0
        """,
    )

    def _get_H2(self):
        """getter of H2"""
        return self._H2

    def _set_H2(self, value):
        """setter of H2"""
        check_var("H2", value, "float", Vmin=0)
        self._H2 = value

    H2 = property(
        fget=_get_H2,
        fset=_set_H2,
        doc=u"""Slot bottom height

        :Type: float
        :min: 0
        """,
    )

    def _get_W4(self):
        """getter of W4"""
        return self._W4

    def _set_W4(self, value):
        """setter of W4"""
        check_var("W4", value, "float", Vmin=0)
        self._W4 = value

    W4 = property(
        fget=_get_W4,
        fset=_set_W4,
        doc=u"""Teeth top width

        :Type: float
        :min: 0
        """,
    )

    def _get_H1(self):
        """getter of H1"""
        return self._H1

    def _set_H1(self, value):
        """setter of H1"""
        check_var("H1", value, "float", Vmin=0)
        self._H1 = value

    H1 = property(
        fget=_get_H1,
        fset=_set_H1,
        doc=u"""Slot top height

        :Type: float
        :min: 0
        """,
    )



# -*- coding: utf-8 -*-
from pyleecan.Classes.CellMat import CellMat
from pyleecan.Classes.MeshMat import MeshMat
from pyleecan.Classes.PointMat import PointMat


def get_mesh(self, label=None, index=None):
    """Return the mesh corresponding to label or an index.

    Parameters
    ----------
    self : MeshSolution
        an MeshSolution object
    label : str
        a label
    index : int
        an index
    group : list
        list of indices of part(s) of the mesh

    Returns
    -------
    mesh: Mesh
        a Mesh object

    """

    if self.is_same_mesh:
        tmpmesh = self.mesh[0]
    else:
        if index is None:
            index = 0
            if label is not None:
                for i, mesh in enumerate(self.mesh):
                    if mesh.label == label:
                        index = i
        tmpmesh = self.mesh[index]

    return tmpmesh



# -*- coding: utf-8 -*-

from ....Classes.OutForce import OutForce
from ....Methods.Simulation.Input import InputError


def gen_input(self):
    """Generate the input for the structural module (skip force computation)

    Parameters
    ----------
    self : InputForce
        An InputForce object
    """

    output = OutForce()

    if self.P is None:
        raise InputError("ERROR: InForce.P missing")
    if self.P.name is None:
        self.P.name = "Magnetic airgap surface force"
    if self.P.symbol is None:
        self.P.symbol = "P"
    P = self.P.get_data()
    output.P = P

    if self.parent.parent is None:
        raise InputError(
            "ERROR: The Simulation object must be in an Output object to run"
        )
    # Save the Output in the correct place
    self.parent.parent.struct = output



# -*- coding: utf-8 -*-

from ....Classes.Arc1 import Arc1
from ....Classes.Arc3 import Arc3
from ....Classes.Segment import Segment


def set_from_point_list(self, point_list, is_sym=False):
    """Set the line_list from a point list (connected by Segments)

    Parameters
    ----------
    self : SlotUD
        A SlotUD object
    point_list : [complex]
        List of complex coordinates
    is_sym : bool
        True to duplicate the point by symmetries
    """

    # Apply symmetry if needed
    Z_list = point_list.copy()
    if is_sym:
        for point in point_list[::-1]:
            Z_list.append(point.conjugate())

    # Creation of curve
    line_list = list()
    for ii in range(len(Z_list) - 1):
        line_list.append(Segment(Z_list[ii], Z_list[ii + 1]))

    self.line_list = line_list



# -*- coding: utf-8 -*-
"""File generated according to SWSlot/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWSlot.Ui_SWSlot import Ui_SWSlot


class Gen_SWSlot(Ui_SWSlot):
    def setupUi(self, SWSlot):
        """Abstract class to update the widget according to the csv doc"""
        Ui_SWSlot.setupUi(self, SWSlot)
        # Setup of in_Zs
        txt = self.tr(u"""slot number""")
        self.in_Zs.setWhatsThis(txt)
        self.in_Zs.setToolTip(txt)

        # Setup of si_Zs
        self.si_Zs.setMinimum(0)
        self.si_Zs.setMaximum(1000)
        txt = self.tr(u"""slot number""")
        self.si_Zs.setWhatsThis(txt)
        self.si_Zs.setToolTip(txt)



from numpy import arcsin, exp, pi, sqrt

from ....Functions.Geometry.inter_line_circle import inter_line_circle


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotDC
        A SlotDC object

    Returns
    -------
    point_list: list
        A list of 15 Point

    """

    Rbo = self.get_Rbo()

    # alpha is the angle to rotate Z0 so ||Z1,Z12|| = W1
    # alpha = self.comp_angle_opening()
    alpha = 2 * float(arcsin(self.W1 / (2 * Rbo)))

    # Compute all points coordinates
    Z0 = Rbo * exp(1j * 0)
    Z1 = Rbo * exp(-1j * alpha / 2)
    if self.is_outwards():
        sign = +1
    else:
        sign = -1

    Zc1 = Z1.real + sign * self.H1
    Zc2 = Zc1 + sign * self.H2
    Zc3 = Zc2 + sign * self.H3

    Z2 = Zc1 - sign * sqrt((self.D1 / 2) ** 2 - (self.W1 / 2) ** 2) - 1j * (self.W1 / 2)
    Z3 = Zc1 + sign * sqrt((self.D1 / 2) ** 2 - (self.W2 / 2) ** 2) - 1j * (self.W2 / 2)
    Z4 = Zc2 - sign * sqrt((self.D2 / 2) ** 2 - (self.W2 / 2) ** 2) - 1j * (self.W2 / 2)

    Z5 = Zc2.real - 1j * self.D2 / 2
    Z6 = Zc3.real - 1j * self.R3

    # Symmetries
    Z7 = Z6.conjugate()
    Z8 = Z5.conjugate()
    Z9 = Z4.conjugate()
    Z10 = Z3.conjugate()
    Z11 = Z2.conjugate()
    Z12 = Z1.conjugate()

    return [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9, Z10, Z11, Z12, Zc1, Zc2, Zc3]



# -*- coding: utf-8 -*-
from SciDataTool import VectorField
from .....Functions.Plot import fft_dict


def get_fund_harm(self, data):
    """Return the fundamental harmonic of the physical quantity in the data object

    Parameters
    ----------
    self : Output
        an Output object
    data : Data
        a Data object

    Returns
    -------
    fund_harm: dict
        Dict containing axis name as key and frequency/order/wavenumber of fundamental harmonic as value

    """

    if data.symbol in ["B", "P", "Is", "Phi_{wind}"]:

        # Init output dict
        fund_harm = dict()

        axes_list = data.get_axes()

        # Get machine pole pair number
        p = self.simu.machine.get_pole_pair_number()

        # Get electrical fundamental frequency
        f_elec = self.simu.input.comp_felec()

        # Loop on axes to express the fundamental harmonic of the Data object
        # including normalizations
        for axe in axes_list:
            # Init fundamental value to None for current axis
            coeff = None

            # Search if the current axis in SciDataTool axis dictionnary
            if axe.name in fft_dict.keys():
                # If yes, find the axis name of the fft
                axe_fft = fft_dict[axe.name]

                # Assign fundamental value depending on axis name
                if axe.name == "time":
                    coeff = f_elec
                elif axe.name == "angle":
                    coeff = p

                # Add in fund_harm dict the values with different normalizations
                if coeff is not None:
                    # Assign fundamental value depending on physical quantity
                    if data.symbol == "P":
                        coeff = 2 * coeff
                    # Store value in dict
                    fund_harm[axe_fft] = coeff

                    # Add also normalizations in dict
                    for key, val in axe.normalizations.items():
                        fund_harm[key] = fund_harm[axe_fft] / val

    # Cannot calculate dict of fundamental harmonic values
    else:
        fund_harm = None

    return fund_harm



# -*- coding: utf-8 -*-


def get_m_name(self):
    """Return the name of the length unit ([m] by default)

    Parameters
    ----------
    self : Unit
        A Unit object

    Returns
    -------
    unit_name: str
        Name of the current unit
    """
    if self.unit_m == 1:
        return "mm"
    else:
        return "m"



from ....Classes.DataKeeper import DataKeeper
from ....Classes.InputCurrent import InputCurrent


def gen_datakeeper_list(self):
    """Generate default DataKeepers for VarLoadSpeed workflow"""
    datakeeper_list = []

    simu = self.parent
    symbol_list = [dk.symbol for dk in self.datakeeper_list]
    # Save speed
    if "N0" not in symbol_list:
        datakeeper_list.append(
            DataKeeper(
                name="Speed",
                symbol="N0",
                unit="rpm",
                keeper="lambda output: output.elec.N0",
            )
        )

    # Get default datakeeper
    if simu and (simu.elec or isinstance(simu.input, InputCurrent)):
        datakeeper_list.extend(self.get_elec_datakeeper(symbol_list))
    if simu and simu.mag:
        datakeeper_list.extend(self.get_mag_datakeeper(symbol_list))
    if simu and simu.force:
        datakeeper_list.extend(self.get_force_datakeeper(symbol_list))

    self.datakeeper_list.extend(datakeeper_list)



# -*- coding: utf-8 -*-

from numpy import arcsin, pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.SlotW24 import *


def check(self):
    """Check that the SlotW24 object is correct

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object

    Returns
    -------
    None

    Raises
    -------
    S24_W3CheckError
        The teeth are too wide, reduce Zs or W3
    S24_HWCheckError
        The teeth are too wide, reduce Zs, H2 or W3
    """
    Rbo = self.get_Rbo()

    if 2 * arcsin(self.W3 / (2 * Rbo)) > 2 * pi / self.Zs:
        raise S24_W3CheckError("The teeth are too wide, reduce Zs or W3")

    if (
        not self.is_outwards()
        and 2 * arcsin(self.W3 / (2 * Rbo - self.H2)) > 2 * pi / self.Zs
    ):
        raise S24_HWCheckError("The teeth are too wide, reduce Zs, H2 or W3")



# -*- coding: utf-8 -*-
from numpy import take
from SciDataTool import Data1D


def get_solution(self, indice=None):
    """Return a copy of the solution with the option to only include specified indice.

    Parameters
    ----------
    self : SolutionData
        a SolutionData object
    indice : list
        list of indice, if list is empty or None all indice are included

    Returns
    -------
    solution: SolutionMat
        solution

    """
    logger = self.get_logger()

    # create copy to directly manipulate data
    solution = self.copy()

    if indice:
        axes = solution.field.axes
        axes_names = solution.get_axes_list()[0]
        ax_idx = axes_names.index("indice")

        org_indice = axes[ax_idx].get_values()

        if set(indice) - set(org_indice):
            logger.warning(
                "At least one input indice is not part of the solution. "
                + "Respective indice will be skipped."
            )

        # skip indice that are not part of the solution
        new_indice = [ii for ii in indice if ii in org_indice]

        # create requested axes list to get field values (see SciDataTool slicing ref.)
        args = [name for name in axes_names]
        args[ax_idx] += "=axis_data"

        # get the field values
        field_dict = solution.field.get_along(*args, axis_data={"indice": new_indice})

        # set new indice axis
        axes[ax_idx] = Data1D(
            values=new_indice,
            is_components=axes[ax_idx].is_components,
            symmetries=axes[ax_idx].symmetries,
            symbol=axes[ax_idx].symbol,
            name=axes[ax_idx].name,
            unit=axes[ax_idx].unit,
            normalizations=axes[ax_idx].normalizations,
        )

        # set new field data
        solution.field.values = field_dict[self.field.symbol]

    return solution



# -*- coding: utf-8 -*-

import PySide2.QtCore
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QWidget

from ......Classes.SlotW26 import SlotW26
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SWSlot.PWSlot26.Gen_PWSlot26 import Gen_PWSlot26
from ......Methods.Slot.Slot import SlotCheckError

translate = PySide2.QtCore.QCoreApplication.translate


class PWSlot26(Gen_PWSlot26, QWidget):
    """Page to set the Slot Type 26"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for Slot combobox
    slot_name = "Slot Type 26"
    slot_type = SlotW26

    def __init__(self, lamination=None):
        """Initialize the GUI according to current lamination

        Parameters
        ----------
        self : PWSlot26
            A PWSlot26 widget
        lamination : Lamination
            current lamination to edit
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        self.lamination = lamination
        self.slot = lamination.slot
        # Set FloatEdit unit
        self.lf_W0.unit = "m"
        self.lf_H0.unit = "m"
        self.lf_H1.unit = "m"
        self.lf_R1.unit = "m"
        self.lf_R2.unit = "m"
        # Set unit name (m ou mm)
        wid_list = [
            self.unit_W0,
            self.unit_H0,
            self.unit_H1,
            self.unit_R1,
            self.unit_R2,
        ]
        for wid in wid_list:
            wid.setText("[" + gui_option.unit.get_m_name() + "]")
        # Fill the fields with the machine values (if they're filled)
        self.lf_W0.setValue(self.slot.W0)
        self.lf_H0.setValue(self.slot.H0)
        self.lf_H1.setValue(self.slot.H1)
        self.lf_R1.setValue(self.slot.R1)
        self.lf_R2.setValue(self.slot.R2)

        # Display the main output of the slot (surface, height...)
        self.w_out.comp_output()

        # Connect the signal
        self.lf_W0.editingFinished.connect(self.set_W0)
        self.lf_H0.editingFinished.connect(self.set_H0)
        self.lf_H1.editingFinished.connect(self.set_H1)
        self.lf_R1.editingFinished.connect(self.set_R1)
        self.lf_R2.editingFinished.connect(self.set_R2)

    def set_W0(self):
        """Signal to update the value of W0 according to the line edit

        Parameters
        ----------
        self : PWSlot26
            A PWSlot26 object
        """
        self.slot.W0 = self.lf_W0.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_H0(self):
        """Signal to update the value of H0 according to the line edit

        Parameters
        ----------
        self : PWSlot26
            A PWSlot26 object
        """
        self.slot.H0 = self.lf_H0.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_H1(self):
        """Signal to update the value of H1 according to the line edit

        Parameters
        ----------
        self : PWSlot26
            A PWSlot26 object
        """
        self.slot.H1 = self.lf_H1.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_R1(self):
        """Signal to update the value of R1 according to the line edit

        Parameters
        ----------
        self : PWSlot26
            A PWSlot26 object
        """
        self.slot.R1 = self.lf_R1.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_R2(self):
        """Signal to update the value of R2 according to the line edit

        Parameters
        ----------
        self : PWSlot26
            A PWSlot26 object
        """
        self.slot.R2 = self.lf_R2.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    @staticmethod
    def check(lam):
        """Check that the current lamination have all the needed field set

        Parameters
        ----------
        lam: LamSlotWind
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """

        # Check that everything is set
        if lam.slot.W0 is None:
            return "You must set W0 !"
        elif lam.slot.H0 is None:
            return "You must set H0 !"
        elif lam.slot.H1 is None:
            return "You must set H1 !"
        elif lam.slot.R1 is None:
            return "You must set R1 !"
        elif lam.slot.R2 is None:
            return "You must set R2 !"
        # Check that everything is set right
        # Constraints
        try:
            lam.slot.check()
        except SlotCheckError as error:
            return str(error)

        # Output
        try:
            yoke_height = lam.comp_height_yoke()
        except Exception as error:
            return "Unable to compute yoke height:" + str(error)
        if yoke_height <= 0:
            return "The slot height is greater than the lamination !"



def insert(self, i, x):
    """
    Insert an item at a given position.
    The first argument is the index of the element before which to insert,
    so a.insert(0, x) inserts at the front of the list, and a.insert(len(a), x) is equivalent to a.append(x).
    """
    return self.output_list.insert(i, x)



# -*- coding: utf-8 -*-
import numpy as np


def add_cell(self, pt_indice, cell_type, group_name=None):
    """Add a new cell defined by a point indices

    Parameters
    ----------
    self : MeshMat
        an Mesh object
    pt_indice : ndarray
        a ndarray of points indices
    group_name : str
        name of the group

    Returns
    -------
    new_ind : int
        Tag of the created element. None if the element already exist
    """

    # Create the new element
    new_ind = 0
    for key in self.cell:  # There should only one solution
        if self.cell[key].indice is not None and self.cell[key].indice.size > 0:
            tmp_ind = max(self.cell[key].indice)
            new_ind = max(new_ind, tmp_ind)
            new_ind += 1

    test_exist = self.cell[cell_type].add_cell(pt_indice, new_ind)

    if test_exist:
        # if group_name in self.group:
        #     self.group[group_name] = np.append(self.group[group_name], new_ind)
        # else:
        #     self.group[group_name] = np.array([new_ind])

        return new_ind
    else:
        return None



# -*- coding: utf-8 -*-

from ....Classes.LamSlotMag import LamSlotMag
from ....Classes.LamSlotWind import LamSlotWind
from ....Methods.Machine import MachineCheckError


def check(self):
    """Check that the Machine object is correct

    Parameters
    ----------
    self : MachineSIPMSM
        A MachineSIPMSM object

    Returns
    -------
    None

    Raises
    _______
    M2C_WrongRotor
        The Rotor of a Machine_Type_2 must be a LamSlotMag
    M2C_WrongStator
        The Stator of a Machine_Type_2 must be a LamSlotWind
    M2C_PError
        The stator and the rotor must have the same value for p
    M2C_MagnetTooLarge
        The airgap is too small for the magnet
    """

    super(type(self), self).check()

    if not isinstance(self.rotor, LamSlotMag):
        raise M2C_WrongRotor("The Rotor of a Machine_Type_2 must be a " "LamSlotMag")
    if not isinstance(self.stator, LamSlotWind):
        raise M2C_WrongStator("The Stator of a Machine_Type_2 must be a " "LamSlotWind")

    if self.rotor.get_pole_pair_number() != self.stator.get_pole_pair_number():
        raise M2C_PError("The stator and the rotor must have the same value " "for p")

    if self.rotor.is_internal and (
        self.rotor.w_slot.comp_radius_mid_active(self.rotor.Rext)
        + self.rotor.Hscr / 2.0
        > self.stator.Rint
    ):
        raise M2C_MagnetTooLarge("The airgap is too small for the magnet")


class M2C_WrongRotor(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    Magnet


    """

    pass


class M2C_WrongStator(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    with
        Winding

    """

    pass


class M2C_PError(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    values


    """

    pass


class M2C_MagnetTooLarge(MachineCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-

from ....Functions.Electrical.comp_fluxlinkage import comp_fluxlinkage as comp_flx
from numpy import zeros, split, mean
import matplotlib.pyplot as plt


def comp_fluxlinkage(self, output):
    """Compute using FEMM the flux linkage

    Parameters
    ----------
    self : FluxLinkFEMM
        a FluxLinkFEMM object
    output : Output
        an Output object
    """

    self.get_logger().info("INFO: Compute flux linkage with FEMM")

    # store orignal currents
    Is = output.elec.Is
    Id_ref = output.elec.Id_ref
    Iq_ref = output.elec.Iq_ref

    # Set currents at 0A for the FEMM simulation
    output.elec.Is = None
    output.elec.Id_ref = 0
    output.elec.Iq_ref = 0

    # compute the fluxlinkage
    fluxdq = comp_flx(self, output)

    # flux = split(Phi_wind, 3, axis=1)
    # fig = plt.figure()
    # plt.plot(angle, flux[0], color="tab:blue", label="A")
    # plt.plot(angle, flux[1], color="tab:red", label="B")
    # plt.plot(angle, flux[2], color="tab:olive", label="C")
    # plt.plot(angle, fluxdq[0], color="k", label="D")
    # plt.plot(angle, fluxdq[1], color="g", label="Q")
    # plt.legend()
    # fig.savefig("test_fluxlink.png")

    # restore orignal currents
    output.elec.Is = Is
    output.elec.Id_ref = Id_ref
    output.elec.Iq_ref = Iq_ref

    return mean(fluxdq[0])



# -*- coding: utf-8 -*-
"""File generated according to DMatLib/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMatLib.Ui_DMatLib import Ui_DMatLib


class Gen_DMatLib(Ui_DMatLib):
    def setupUi(self, DMatLib):
        """Abstract class to update the widget according to the csv doc"""
        Ui_DMatLib.setupUi(self, DMatLib)
        # Setup of out_name
        txt = self.tr(u"""name of the material""")
        self.out_name.setWhatsThis(txt)
        self.out_name.setToolTip(txt)

        # Setup of out_iso
        txt = self.tr(u"""If True, uniformity in all orientations""")
        self.out_iso.setWhatsThis(txt)
        self.out_iso.setToolTip(txt)

        # Setup of out_rho_elec
        txt = self.tr(u"""Resistivity at 20 deg C""")
        self.out_rho_elec.setWhatsThis(txt)
        self.out_rho_elec.setToolTip(txt)

        # Setup of out_epsr
        txt = self.tr(u"""Relative dielectric constant""")
        self.out_epsr.setWhatsThis(txt)
        self.out_epsr.setToolTip(txt)

        # Setup of out_cost_unit
        txt = self.tr(u"""Cost of one kilo of material""")
        self.out_cost_unit.setWhatsThis(txt)
        self.out_cost_unit.setToolTip(txt)

        # Setup of out_Cp
        txt = self.tr(u"""specific heat capacity""")
        self.out_Cp.setWhatsThis(txt)
        self.out_Cp.setToolTip(txt)

        # Setup of out_alpha
        txt = self.tr(u"""thermal expansion coefficient""")
        self.out_alpha.setWhatsThis(txt)
        self.out_alpha.setToolTip(txt)

        # Setup of out_L
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_L.setWhatsThis(txt)
        self.out_L.setToolTip(txt)

        # Setup of out_LX
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_LX.setWhatsThis(txt)
        self.out_LX.setToolTip(txt)

        # Setup of out_LY
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_LY.setWhatsThis(txt)
        self.out_LY.setToolTip(txt)

        # Setup of out_LZ
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_LZ.setWhatsThis(txt)
        self.out_LZ.setToolTip(txt)

        # Setup of out_rho_meca
        txt = self.tr(u"""mass per unit volume [kg/m3]""")
        self.out_rho_meca.setWhatsThis(txt)
        self.out_rho_meca.setToolTip(txt)

        # Setup of out_E
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_E.setWhatsThis(txt)
        self.out_E.setToolTip(txt)

        # Setup of out_EX
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_EX.setWhatsThis(txt)
        self.out_EX.setToolTip(txt)

        # Setup of out_EY
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_EY.setWhatsThis(txt)
        self.out_EY.setToolTip(txt)

        # Setup of out_EZ
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_EZ.setWhatsThis(txt)
        self.out_EZ.setToolTip(txt)

        # Setup of out_G
        txt = self.tr(
            u"""shear modulus in XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_G.setWhatsThis(txt)
        self.out_G.setToolTip(txt)

        # Setup of out_GXY
        txt = self.tr(
            u"""shear modulus in XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_GXY.setWhatsThis(txt)
        self.out_GXY.setToolTip(txt)

        # Setup of out_GXZ
        txt = self.tr(
            u"""shear modulus in XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_GXZ.setWhatsThis(txt)
        self.out_GXZ.setToolTip(txt)

        # Setup of out_GYZ
        txt = self.tr(
            u"""shear modulus in YZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_GYZ.setWhatsThis(txt)
        self.out_GYZ.setToolTip(txt)

        # Setup of out_nu
        txt = self.tr(
            u"""equivalent Poisson ratio in the XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_nu.setWhatsThis(txt)
        self.out_nu.setToolTip(txt)

        # Setup of out_nu_XY
        txt = self.tr(
            u"""equivalent Poisson ratio in the XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_nu_XY.setWhatsThis(txt)
        self.out_nu_XY.setToolTip(txt)

        # Setup of out_nu_XZ
        txt = self.tr(
            u"""equivalent Poisson ratio in the XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_nu_XZ.setWhatsThis(txt)
        self.out_nu_XZ.setToolTip(txt)

        # Setup of out_nu_YZ
        txt = self.tr(
            u"""equivalent Poisson ratio in the YZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.out_nu_YZ.setWhatsThis(txt)
        self.out_nu_YZ.setToolTip(txt)

        # Setup of out_mur_lin
        txt = self.tr(u"""Relative magnetic permeability""")
        self.out_mur_lin.setWhatsThis(txt)
        self.out_mur_lin.setToolTip(txt)

        # Setup of out_Brm20
        txt = self.tr(u"""magnet remanence induction at 20degC""")
        self.out_Brm20.setWhatsThis(txt)
        self.out_Brm20.setToolTip(txt)

        # Setup of out_alpha_Br
        txt = self.tr(
            u"""temperature coefficient for remanent flux density /degC compared to 20degC"""
        )
        self.out_alpha_Br.setWhatsThis(txt)
        self.out_alpha_Br.setToolTip(txt)

        # Setup of out_wlam
        txt = self.tr(
            u"""lamination sheet width without insulation [m] (0 == not laminated)"""
        )
        self.out_wlam.setWhatsThis(txt)
        self.out_wlam.setToolTip(txt)



# -*- coding: utf-8 -*-

# File generated according to SWindCond.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from .....GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SWindCond(object):
    def setupUi(self, SWindCond):
        if not SWindCond.objectName():
            SWindCond.setObjectName(u"SWindCond")
        SWindCond.resize(650, 550)
        SWindCond.setMinimumSize(QSize(650, 550))
        self.main_layout = QVBoxLayout(SWindCond)
        self.main_layout.setObjectName(u"main_layout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_cond_type = QLabel(SWindCond)
        self.in_cond_type.setObjectName(u"in_cond_type")

        self.horizontalLayout.addWidget(self.in_cond_type)

        self.c_cond_type = QComboBox(SWindCond)
        self.c_cond_type.addItem("")
        self.c_cond_type.addItem("")
        self.c_cond_type.setObjectName(u"c_cond_type")
        self.c_cond_type.setMinimumSize(QSize(150, 0))

        self.horizontalLayout.addWidget(self.c_cond_type)

        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.w_mat_0 = WMatSelect(SWindCond)
        self.w_mat_0.setObjectName(u"w_mat_0")
        self.w_mat_0.setMinimumSize(QSize(100, 0))

        self.verticalLayout.addWidget(self.w_mat_0)

        self.w_mat_1 = WMatSelect(SWindCond)
        self.w_mat_1.setObjectName(u"w_mat_1")
        self.w_mat_1.setMinimumSize(QSize(100, 0))

        self.verticalLayout.addWidget(self.w_mat_1)

        self.horizontalLayout.addLayout(self.verticalLayout)

        self.horizontalSpacer_3 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer_3)

        self.main_layout.addLayout(self.horizontalLayout)

        self.w_cond = QWidget(SWindCond)
        self.w_cond.setObjectName(u"w_cond")

        self.main_layout.addWidget(self.w_cond)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.b_previous = QPushButton(SWindCond)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout_2.addWidget(self.b_previous)

        self.b_next = QPushButton(SWindCond)
        self.b_next.setObjectName(u"b_next")

        self.horizontalLayout_2.addWidget(self.b_next)

        self.main_layout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(SWindCond)

        QMetaObject.connectSlotsByName(SWindCond)

    # setupUi

    def retranslateUi(self, SWindCond):
        SWindCond.setWindowTitle(QCoreApplication.translate("SWindCond", u"Form", None))
        self.in_cond_type.setText(
            QCoreApplication.translate("SWindCond", u"Conductor type :", None)
        )
        self.c_cond_type.setItemText(
            0, QCoreApplication.translate("SWindCond", u"Preformed Rectangular", None)
        )
        self.c_cond_type.setItemText(
            1, QCoreApplication.translate("SWindCond", u"Random Round Wire", None)
        )

        self.b_previous.setText(
            QCoreApplication.translate("SWindCond", u"Previous", None)
        )
        self.b_next.setText(QCoreApplication.translate("SWindCond", u"Next", None))

    # retranslateUi



# -*- coding: utf-8 -*-

# File generated according to PCondType22.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect
from ......GUI.Dialog.DMachineSetup.SBar.WBarOut.WBarOut import WBarOut


class Ui_PCondType22(object):
    def setupUi(self, PCondType22):
        if not PCondType22.objectName():
            PCondType22.setObjectName(u"PCondType22")
        PCondType22.resize(460, 124)
        self.horizontalLayout = QHBoxLayout(PCondType22)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.w_mat = WMatSelect(PCondType22)
        self.w_mat.setObjectName(u"w_mat")

        self.horizontalLayout.addWidget(self.w_mat)

        self.w_out = WBarOut(PCondType22)
        self.w_out.setObjectName(u"w_out")

        self.horizontalLayout.addWidget(self.w_out)

        self.retranslateUi(PCondType22)

        QMetaObject.connectSlotsByName(PCondType22)

    # setupUi

    def retranslateUi(self, PCondType22):
        PCondType22.setWindowTitle(
            QCoreApplication.translate("PCondType22", u"Form", None)
        )

    # retranslateUi



# -*- coding: utf-8 -*-
from numpy import abs, exp, pi, sum, array, linspace


def comp_winding_factor(self, Harmonics=[1]):
    """Compute the winding factor of phase 1 (asuming symmetry)

    Parameters
    ----------
    self : Winding
        A: Winding object
    Harmonics : list of floats
        list of harmonics to calculate the winding factor

    Returns
    -------
    xi: numpy.ndarray
        winding factor

    Raises
    ------

    """
    if self.parent is None:
        raise WindingError("ERROR: The Winding object must be in a Lamination object.")

    if self.parent.slot is None:
        raise WindingError(
            "ERROR: The Winding object must be in a Lamination object with Slot."
        )

    Zs = self.parent.slot.Zs

    assert Zs > 0, "Zs must be >0"
    assert Zs % 1 == 0, "Zs must be an integer"

    wind_mat = self.comp_connection_mat(Zs)
    p = self.p

    # slot electrical angle
    slot_ang = linspace(0, Zs, num=Zs, endpoint=False) * 2 * pi / Zs * p

    phase = 0
    wind_ph1 = sum(wind_mat[:, :, :, phase], (0, 1))
    wind_ref = sum(abs(wind_mat[:, :, :, phase]))

    xi = list()

    for harm in Harmonics:
        xi.append(abs(sum(exp(1j * slot_ang * harm) * wind_ph1)) / wind_ref)

    xi = array(xi)

    return xi
